# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: accelbyte_cloud_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import HeaderStr
from ....core import get_namespace as get_services_namespace
from ....core import run_request
from ....core import run_request_async
from ....core import same_doc_as

from ..models import ModelsAssignRoleToMemberRequestV1
from ..models import ModelsCreateMemberRoleRequestV1
from ..models import ModelsCreateMemberRoleResponseV1
from ..models import ModelsGetMemberRoleResponseV1
from ..models import ModelsGetMemberRolesListResponseV1
from ..models import ModelsGetUserGroupInformationResponseV1
from ..models import ModelsRemoveRoleFromMemberRequestV1
from ..models import ModelsUpdateMemberRolePermissionsRequestV1
from ..models import ModelsUpdateMemberRoleRequestV1
from ..models import ModelsUpdateMemberRoleResponseV1
from ..models import ResponseErrorResponse

from ..operations.group_roles import CreateMemberRoleAdminV1
from ..operations.group_roles import DeleteMemberRoleAdminV1
from ..operations.group_roles import DeleteMemberRolePublicV1
from ..operations.group_roles import GetMemberRolesListAdminV1
from ..operations.group_roles import GetMemberRolesListPublicV1
from ..operations.group_roles import GetSingleMemberRoleAdminV1
from ..operations.group_roles import UpdateMemberRoleAdminV1
from ..operations.group_roles import UpdateMemberRolePermissionAdminV1
from ..operations.group_roles import UpdateMemberRolePublicV1


@same_doc_as(CreateMemberRoleAdminV1)
def create_member_role_admin_v1(
    body: ModelsCreateMemberRoleRequestV1,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = CreateMemberRoleAdminV1.create(
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(CreateMemberRoleAdminV1)
async def create_member_role_admin_v1_async(
    body: ModelsCreateMemberRoleRequestV1,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = CreateMemberRoleAdminV1.create(
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(DeleteMemberRoleAdminV1)
def delete_member_role_admin_v1(
    member_role_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = DeleteMemberRoleAdminV1.create(
        member_role_id=member_role_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(DeleteMemberRoleAdminV1)
async def delete_member_role_admin_v1_async(
    member_role_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = DeleteMemberRoleAdminV1.create(
        member_role_id=member_role_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(DeleteMemberRolePublicV1)
def delete_member_role_public_v1(
    body: ModelsRemoveRoleFromMemberRequestV1,
    member_role_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = DeleteMemberRolePublicV1.create(
        body=body,
        member_role_id=member_role_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(DeleteMemberRolePublicV1)
async def delete_member_role_public_v1_async(
    body: ModelsRemoveRoleFromMemberRequestV1,
    member_role_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = DeleteMemberRolePublicV1.create(
        body=body,
        member_role_id=member_role_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(GetMemberRolesListAdminV1)
def get_member_roles_list_admin_v1(
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetMemberRolesListAdminV1.create(
        limit=limit,
        offset=offset,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(GetMemberRolesListAdminV1)
async def get_member_roles_list_admin_v1_async(
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetMemberRolesListAdminV1.create(
        limit=limit,
        offset=offset,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(GetMemberRolesListPublicV1)
def get_member_roles_list_public_v1(
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetMemberRolesListPublicV1.create(
        limit=limit,
        offset=offset,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(GetMemberRolesListPublicV1)
async def get_member_roles_list_public_v1_async(
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetMemberRolesListPublicV1.create(
        limit=limit,
        offset=offset,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(GetSingleMemberRoleAdminV1)
def get_single_member_role_admin_v1(
    member_role_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetSingleMemberRoleAdminV1.create(
        member_role_id=member_role_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(GetSingleMemberRoleAdminV1)
async def get_single_member_role_admin_v1_async(
    member_role_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetSingleMemberRoleAdminV1.create(
        member_role_id=member_role_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(UpdateMemberRoleAdminV1)
def update_member_role_admin_v1(
    body: ModelsUpdateMemberRoleRequestV1,
    member_role_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = UpdateMemberRoleAdminV1.create(
        body=body,
        member_role_id=member_role_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(UpdateMemberRoleAdminV1)
async def update_member_role_admin_v1_async(
    body: ModelsUpdateMemberRoleRequestV1,
    member_role_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = UpdateMemberRoleAdminV1.create(
        body=body,
        member_role_id=member_role_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(UpdateMemberRolePermissionAdminV1)
def update_member_role_permission_admin_v1(
    body: ModelsUpdateMemberRolePermissionsRequestV1,
    member_role_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = UpdateMemberRolePermissionAdminV1.create(
        body=body,
        member_role_id=member_role_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(UpdateMemberRolePermissionAdminV1)
async def update_member_role_permission_admin_v1_async(
    body: ModelsUpdateMemberRolePermissionsRequestV1,
    member_role_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = UpdateMemberRolePermissionAdminV1.create(
        body=body,
        member_role_id=member_role_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(UpdateMemberRolePublicV1)
def update_member_role_public_v1(
    body: ModelsAssignRoleToMemberRequestV1,
    member_role_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = UpdateMemberRolePublicV1.create(
        body=body,
        member_role_id=member_role_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(UpdateMemberRolePublicV1)
async def update_member_role_public_v1_async(
    body: ModelsAssignRoleToMemberRequestV1,
    member_role_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = UpdateMemberRolePublicV1.create(
        body=body,
        member_role_id=member_role_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: accelbyte_cloud_py_codegen

# AccelByte Cloud Iam Service (5.22.0)

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model

from ..models.model_platform_user_information import ModelPlatformUserInformation


class ModelUserInformation(Model):
    """Model user information (model.UserInformation)

    Properties:
        country: (Country) REQUIRED str

        display_name: (DisplayName) REQUIRED str

        email_addresses: (EmailAddresses) REQUIRED List[str]

        linked_platform_accounts: (LinkedPlatformAccounts) REQUIRED List[ModelPlatformUserInformation]

        phone_number: (PhoneNumber) REQUIRED str

        username: (Username) REQUIRED str

        xuid: (XUID) OPTIONAL str
    """

    # region fields

    country: str  # REQUIRED
    display_name: str  # REQUIRED
    email_addresses: List[str]  # REQUIRED
    linked_platform_accounts: List[ModelPlatformUserInformation]  # REQUIRED
    phone_number: str  # REQUIRED
    username: str  # REQUIRED
    xuid: str  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_country(self, value: str) -> ModelUserInformation:
        self.country = value
        return self

    def with_display_name(self, value: str) -> ModelUserInformation:
        self.display_name = value
        return self

    def with_email_addresses(self, value: List[str]) -> ModelUserInformation:
        self.email_addresses = value
        return self

    def with_linked_platform_accounts(
        self, value: List[ModelPlatformUserInformation]
    ) -> ModelUserInformation:
        self.linked_platform_accounts = value
        return self

    def with_phone_number(self, value: str) -> ModelUserInformation:
        self.phone_number = value
        return self

    def with_username(self, value: str) -> ModelUserInformation:
        self.username = value
        return self

    def with_xuid(self, value: str) -> ModelUserInformation:
        self.xuid = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "country"):
            result["Country"] = str(self.country)
        elif include_empty:
            result["Country"] = ""
        if hasattr(self, "display_name"):
            result["DisplayName"] = str(self.display_name)
        elif include_empty:
            result["DisplayName"] = ""
        if hasattr(self, "email_addresses"):
            result["EmailAddresses"] = [str(i0) for i0 in self.email_addresses]
        elif include_empty:
            result["EmailAddresses"] = []
        if hasattr(self, "linked_platform_accounts"):
            result["LinkedPlatformAccounts"] = [
                i0.to_dict(include_empty=include_empty)
                for i0 in self.linked_platform_accounts
            ]
        elif include_empty:
            result["LinkedPlatformAccounts"] = []
        if hasattr(self, "phone_number"):
            result["PhoneNumber"] = str(self.phone_number)
        elif include_empty:
            result["PhoneNumber"] = ""
        if hasattr(self, "username"):
            result["Username"] = str(self.username)
        elif include_empty:
            result["Username"] = ""
        if hasattr(self, "xuid"):
            result["XUID"] = str(self.xuid)
        elif include_empty:
            result["XUID"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        country: str,
        display_name: str,
        email_addresses: List[str],
        linked_platform_accounts: List[ModelPlatformUserInformation],
        phone_number: str,
        username: str,
        xuid: Optional[str] = None,
    ) -> ModelUserInformation:
        instance = cls()
        instance.country = country
        instance.display_name = display_name
        instance.email_addresses = email_addresses
        instance.linked_platform_accounts = linked_platform_accounts
        instance.phone_number = phone_number
        instance.username = username
        if xuid is not None:
            instance.xuid = xuid
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ModelUserInformation:
        instance = cls()
        if not dict_:
            return instance
        if "Country" in dict_ and dict_["Country"] is not None:
            instance.country = str(dict_["Country"])
        elif include_empty:
            instance.country = ""
        if "DisplayName" in dict_ and dict_["DisplayName"] is not None:
            instance.display_name = str(dict_["DisplayName"])
        elif include_empty:
            instance.display_name = ""
        if "EmailAddresses" in dict_ and dict_["EmailAddresses"] is not None:
            instance.email_addresses = [str(i0) for i0 in dict_["EmailAddresses"]]
        elif include_empty:
            instance.email_addresses = []
        if (
            "LinkedPlatformAccounts" in dict_
            and dict_["LinkedPlatformAccounts"] is not None
        ):
            instance.linked_platform_accounts = [
                ModelPlatformUserInformation.create_from_dict(
                    i0, include_empty=include_empty
                )
                for i0 in dict_["LinkedPlatformAccounts"]
            ]
        elif include_empty:
            instance.linked_platform_accounts = []
        if "PhoneNumber" in dict_ and dict_["PhoneNumber"] is not None:
            instance.phone_number = str(dict_["PhoneNumber"])
        elif include_empty:
            instance.phone_number = ""
        if "Username" in dict_ and dict_["Username"] is not None:
            instance.username = str(dict_["Username"])
        elif include_empty:
            instance.username = ""
        if "XUID" in dict_ and dict_["XUID"] is not None:
            instance.xuid = str(dict_["XUID"])
        elif include_empty:
            instance.xuid = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ModelUserInformation]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ModelUserInformation]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        ModelUserInformation,
        List[ModelUserInformation],
        Dict[Any, ModelUserInformation],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "Country": "country",
            "DisplayName": "display_name",
            "EmailAddresses": "email_addresses",
            "LinkedPlatformAccounts": "linked_platform_accounts",
            "PhoneNumber": "phone_number",
            "Username": "username",
            "XUID": "xuid",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "Country": True,
            "DisplayName": True,
            "EmailAddresses": True,
            "LinkedPlatformAccounts": True,
            "PhoneNumber": True,
            "Username": True,
            "XUID": False,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: accelbyte_cloud_py_codegen

"""Auto-generated package that contains models used by the AccelByte Cloud Ugc Service."""

__version__ = "2.6.0"
__author__ = "AccelByte"
__email__ = "dev@accelbyte.net"

# pylint: disable=line-too-long

from .models_add_download_count_response import ModelsAddDownloadCountResponse
from .models_channel_request import ModelsChannelRequest
from .models_channel_response import ModelsChannelResponse
from .models_content_download_response import ModelsContentDownloadResponse
from .models_content_like_request import ModelsContentLikeRequest
from .models_content_like_response import ModelsContentLikeResponse
from .models_create_content_request import ModelsCreateContentRequest
from .models_create_content_request_s3 import ModelsCreateContentRequestS3
from .models_create_content_response import ModelsCreateContentResponse
from .models_create_group_request import ModelsCreateGroupRequest
from .models_create_group_response import ModelsCreateGroupResponse
from .models_create_screenshot_request import ModelsCreateScreenshotRequest
from .models_create_screenshot_request_item import ModelsCreateScreenshotRequestItem
from .models_create_screenshot_request_item import (
    FileExtensionEnum as ModelsCreateScreenshotRequestItemFileExtensionEnum,
)
from .models_create_screenshot_response import ModelsCreateScreenshotResponse
from .models_create_tag_request import ModelsCreateTagRequest
from .models_create_tag_response import ModelsCreateTagResponse
from .models_create_type_request import ModelsCreateTypeRequest
from .models_create_type_response import ModelsCreateTypeResponse
from .models_creator_follow_state import ModelsCreatorFollowState
from .models_creator_overview_response import ModelsCreatorOverviewResponse
from .models_creator_response import ModelsCreatorResponse
from .models_get_content_preview_response import ModelsGetContentPreviewResponse
from .models_hide_content_request import ModelsHideContentRequest
from .models_like_state import ModelsLikeState
from .models_paginated_content_download_response import (
    ModelsPaginatedContentDownloadResponse,
)
from .models_paginated_creator_overview_response import (
    ModelsPaginatedCreatorOverviewResponse,
)
from .models_paginated_get_channel_response import ModelsPaginatedGetChannelResponse
from .models_paginated_get_tag_response import ModelsPaginatedGetTagResponse
from .models_paginated_get_type_response import ModelsPaginatedGetTypeResponse
from .models_paginated_group_response import ModelsPaginatedGroupResponse
from .models_paging_cursor import ModelsPagingCursor
from .models_payload_url import ModelsPayloadURL
from .models_preview_metadata import ModelsPreviewMetadata
from .models_preview_url import ModelsPreviewURL
from .models_public_get_content_bulk_request import ModelsPublicGetContentBulkRequest
from .models_screenshot_response import ModelsScreenshotResponse
from .models_update_screenshot import ModelsUpdateScreenshot
from .models_update_screenshot_request import ModelsUpdateScreenshotRequest
from .models_update_screenshot_response import ModelsUpdateScreenshotResponse
from .models_user_follow_request import ModelsUserFollowRequest
from .models_user_follow_response import ModelsUserFollowResponse
from .response_error import ResponseError



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.

from __future__ import annotations

import logging

from os import PathLike as OSPathLike
from typing import Any, Dict, Iterable, Optional, Protocol, Set, Tuple

from ._config_repository import ConfigRepository, CONFIG_REPOS, DEFAULT_CONFIG_REPO
from ._http_client import HttpClient, HTTP_CLIENTS, DEFAULT_HTTP_CLIENT
from ._http_response import HttpResponse
from ._operation import Operation
from ._proto_http_request import ProtoHttpRequest, create_proto_from_operation
from ._token_repository import TokenRepository, TOKEN_REPOS, DEFAULT_TOKEN_REPO
from ._utils import (
    add_buffered_file_handler_to_logger,
    get_query_from_http_redirect_response,
)


class OperationPreprocessor(Protocol):
    def __call__(
        self, operation: Operation, sdk: AccelByteSDK, *args, **kwargs
    ) -> Operation:
        ...


class RequestPreprocessor(Protocol):
    def __call__(
        self, proto: ProtoHttpRequest, sdk: AccelByteSDK, *args, **kwargs
    ) -> ProtoHttpRequest:
        ...


class ResponsePreprocessor(Protocol):
    def __call__(
        self, response: Any, operation: Operation, sdk: AccelByteSDK, *args, **kwargs
    ) -> Tuple[Any, Optional[HttpResponse]]:
        ...


def extract_redirect_query(
    response: Any, operation: Operation, sdk: AccelByteSDK
) -> Tuple[Any, Optional[HttpResponse]]:
    if (
        isinstance(response, HttpResponse)
        and operation.has_redirects()
        and operation.location_query
    ):
        response, error = get_query_from_http_redirect_response(
            response, operation.location_query
        )
        if error:
            return None, error

    return response, None


class AccelByteSDK:

    LOGGER_NAME: str = "accelbyte_py_sdk"

    def __init__(self):
        self._is_initialized: bool = False
        self._logger_names: Set[str] = {AccelByteSDK.LOGGER_NAME}

        self._config_repository: Optional[ConfigRepository] = None
        self._token_repository: Optional[TokenRepository] = None
        self._http_client: Optional[HttpClient] = None

        self.logger = logging.getLogger(AccelByteSDK.LOGGER_NAME)
        self.operation_preprocessors: Dict[str, OperationPreprocessor] = {}
        self.request_preprocessors: Dict[str, RequestPreprocessor] = {}
        self.response_preprocessors: Dict[str, ResponsePreprocessor] = {
            "extract_redirect_query": extract_redirect_query
        }

    # region Lifecycle

    def is_initialized(self) -> bool:
        return self._is_initialized

    def initialize(
        self,
        /,
        *,
        options: Optional[Dict[str, Any]] = None,
    ) -> None:
        """Initializes the AccelByte Python SDK.

        Args:
            options (dict): Options used by the SDK. It can contain any of the following:
             -- config (Optional[Union[str, Type[ConfigRepository]]]): Config Repository to use.
             -- config_params (Optional[Tuple[List[Any], Dict[str, Any]]]): Config Repository parameters.
             -- token (Optional[Union[str, Type[TokenRepository]]]): Token Repository to use.
             -- token_params (Optional[Tuple[List[Any], Dict[str, Any]]]): Token Repository parameters.
             -- http (Optional[Union[str, Type[HttpClient]]]): Http Client to use.
             -- http_params (Optional[Tuple[List[Any], Dict[str, Any]]]): Http Client parameters.
             -- log_files (Optional[Dict[str, Union[str, os.PathLike, dict]]]): Log files.
        """
        options = options if options is not None else {}

        if self.is_initialized():
            self.logger.warning("Already initialized.")
            return

        # config repository

        if "config" in options:
            config_repo = options.get("config")
            if isinstance(config_repo, str):
                config_params = options.get("config_params")
                (
                    is_config_params_valid,
                    config_args,
                    config_kwargs,
                ) = self.__is_valid_options_params(config_params)
                if not is_config_params_valid:
                    raise ValueError(
                        "Config params must be a Tuple[List[Any], Dict[str, Any]]."
                    )
                config_impl = next(
                    (impl for impl in CONFIG_REPOS if impl.__name__ == config_repo),
                    None,
                )
                if config_impl is None:
                    raise ValueError(
                        f"Config repository '{config_repo}' not recognized."
                    )
                config_repo = config_impl(*config_args, **config_kwargs)
        else:
            config_repo = DEFAULT_CONFIG_REPO()

        self.set_config_repository(config_repo)

        # token repository

        if "token" in options:
            token_repo = options.get("token")
            if isinstance(token_repo, str):
                token_params = options.get("token_params")
                (
                    is_token_params_valid,
                    token_args,
                    token_kwargs,
                ) = self.__is_valid_options_params(token_params)
                if not is_token_params_valid:
                    raise ValueError(
                        "Token params must be a Tuple[List[Any], Dict[str, Any]]."
                    )
                token_impl = next(
                    (impl for impl in TOKEN_REPOS if impl.__name__ == token_repo), None
                )
                if token_impl is None:
                    raise ValueError(f"Token repository '{token_repo}' not recognized.")
                token_repo = token_impl(*token_args, **token_kwargs)
        else:
            token_repo = DEFAULT_TOKEN_REPO()

        self.set_token_repository(token_repo)

        # http client

        if "http" in options:
            http_client = options.get("http")
            if isinstance(http_client, str):
                http_params = options.get("http_params")
                (
                    is_http_params_valid,
                    http_args,
                    http_kwargs,
                ) = self.__is_valid_options_params(http_params)
                if not is_http_params_valid:
                    raise ValueError(
                        "HTTP params must be a Tuple[List[Any], Dict[str, Any]]."
                    )
                http_impl = next(
                    (impl for impl in HTTP_CLIENTS if impl.__name__ == http_client),
                    None,
                )
                if http_impl is None:
                    raise ValueError(f"HTTP client '{http_client}' not recognized.")
                http_client = http_impl(*http_args, **http_kwargs)
        else:
            http_client = DEFAULT_HTTP_CLIENT()

        self.set_http_client(http_client)

        # log files

        if log_files := options.get("log_files"):
            if not isinstance(log_files, dict):
                raise ValueError(
                    "Log files must be a Dict[str, Union[str, os.PathLike, dict]]."
                )
            for log_scope, log_options in log_files.items():
                if not isinstance(log_scope, str):
                    raise ValueError("Log files keys must be str.")
                log_file_kwargs = {}
                if isinstance(log_options, dict):
                    log_file_kwargs = log_options
                elif isinstance(log_options, (str, OSPathLike)):
                    log_file_kwargs["filename"] = log_options
                if "capacity" not in log_file_kwargs:
                    log_file_kwargs["capacity"] = 10
                if log_scope == AccelByteSDK.LOGGER_NAME:
                    log_scope = None
                else:
                    log_scope = log_scope.removeprefix(f"{AccelByteSDK.LOGGER_NAME}.")
                    self._logger_names.add(f"{AccelByteSDK.LOGGER_NAME}.{log_scope}")
                log_file_kwargs["additional_scope"] = log_scope
                add_buffered_file_handler_to_logger(**log_file_kwargs)

        # other

        self._is_initialized = True

        self.logger.info("AccelByte Python SDK initialized.")

    def reset(self) -> None:
        if self._http_client is not None:
            self._http_client.close()

        self.__clear_logger_handlers(self._logger_names)

        self._is_initialized = False
        self._logger_names = {AccelByteSDK.LOGGER_NAME}

        self._config_repository = None
        self._token_repository = None
        self._http_client = None

    # endregion Lifecycle

    # region Accessors

    def get_config_repository(
        self, raise_when_none: bool = True
    ) -> Optional[ConfigRepository]:
        if raise_when_none and self._config_repository is None:
            raise ValueError("Config repository not set.")
        return self._config_repository

    def set_config_repository(self, config_repository: ConfigRepository) -> None:
        if not isinstance(config_repository, ConfigRepository):
            raise TypeError(
                f"Config repository '{type(config_repository).__name__}' not valid."
            )
        self._config_repository = config_repository

    def get_token_repository(
        self, raise_when_none: bool = True
    ) -> Optional[TokenRepository]:
        if raise_when_none and self._token_repository is None:
            raise ValueError("Token repository not set.")
        return self._token_repository

    def set_token_repository(self, token_repository: TokenRepository) -> None:
        if not isinstance(token_repository, TokenRepository):
            raise TypeError(
                f"Token repository '{type(token_repository).__name__}' not valid."
            )
        self._token_repository = token_repository

    def get_http_client(self, raise_when_none: bool = True) -> Optional[HttpClient]:
        if raise_when_none and self._http_client is None:
            raise ValueError("HTTP client not set.")
        return self._http_client

    def set_http_client(self, http_client: HttpClient) -> None:
        if not isinstance(http_client, HttpClient):
            raise TypeError(f"HTTP client '{type(http_client).__name__}' not valid.")
        self._http_client = http_client

    # endregion Accessors

    # region Ease of Access (result, error)

    def get_app_name(self) -> Tuple[Optional[str], Optional[HttpResponse]]:
        if config_repo := self.get_config_repository(raise_when_none=False):
            app_name = config_repo.get_app_name()
            if not app_name:
                return None, HttpResponse.create_error(400, "App name not found.")
            return app_name, None
        return None, HttpResponse.create_config_repo_not_found_error()

    def get_app_version(self) -> Tuple[Optional[str], Optional[HttpResponse]]:
        if config_repo := self.get_config_repository(raise_when_none=False):
            app_version = config_repo.get_app_version()
            if not app_version:
                return None, HttpResponse.create_error(400, "App version not found.")
            return app_version, None
        return None, HttpResponse.create_config_repo_not_found_error()

    def get_base_url(self) -> Tuple[Optional[str], Optional[HttpResponse]]:
        if config_repo := self.get_config_repository(raise_when_none=False):
            if base_url := config_repo.get_base_url():
                return base_url, None
            return None, HttpResponse.create_base_url_not_set_error()
        return None, HttpResponse.create_config_repo_not_found_error()

    def get_client_auth(
        self,
    ) -> Tuple[Optional[Tuple[str, str]], Optional[HttpResponse]]:
        if config_repo := self.get_config_repository(raise_when_none=False):
            client_id = config_repo.get_client_id()
            if not client_id:
                return None, HttpResponse.create_client_not_registered_error()
            client_secret = config_repo.get_client_secret() or ""
            return (client_id, client_secret), None
        return None, HttpResponse.create_config_repo_not_found_error()

    def get_client_id(self) -> Tuple[Optional[str], Optional[HttpResponse]]:
        if config_repo := self.get_config_repository(raise_when_none=False):
            client_id = config_repo.get_client_id()
            if not client_id:
                return None, HttpResponse.create_client_not_registered_error()
            return client_id, None
        return None, HttpResponse.create_config_repo_not_found_error()

    def get_client_secret(self) -> Tuple[Optional[str], Optional[HttpResponse]]:
        if config_repo := self.get_config_repository(raise_when_none=False):
            client_secret = config_repo.get_client_secret() or ""
            return client_secret, None
        return None, HttpResponse.create_config_repo_not_found_error()

    def get_namespace(self) -> Tuple[Optional[str], Optional[HttpResponse]]:
        if config_repo := self.get_config_repository(raise_when_none=False):
            namespace = config_repo.get_namespace()
            if not namespace:
                return None, HttpResponse.create_error(400, "Namespace not found.")
            return namespace, None
        return None, HttpResponse.create_config_repo_not_found_error()

    def get_access_token(self) -> Tuple[Optional[str], Optional[HttpResponse]]:
        if token_repo := self.get_token_repository(raise_when_none=False):
            access_token = token_repo.get_access_token()
            if not access_token:
                return None, HttpResponse.create_token_not_found_error()
            return access_token, None
        return None, HttpResponse.create_token_repo_not_found_error()

    def remove_token(self) -> Tuple[None, Optional[HttpResponse]]:
        if token_repo := self.get_token_repository(raise_when_none=False):
            success = token_repo.remove_token()
            if not success:
                return None, HttpResponse.create_failed_to_remove_token_error()
            return None, None
        return None, HttpResponse.create_token_repo_not_found_error()

    def set_token(self, token: Any) -> Tuple[None, Optional[HttpResponse]]:
        if token_repo := self.get_token_repository(raise_when_none=False):
            success = token_repo.store_token(token)
            if not success:
                return None, HttpResponse.create_failed_to_set_token_error()
            return None, None
        return None, HttpResponse.create_token_repo_not_found_error()

    # endregion Ease of Access (result, error)

    # region Requests

    def run_request(self, operation: Operation, **kwargs) -> Tuple[Any, Any]:

        proto, error, kwargs = self._pre_run_request(operation=operation, **kwargs)
        if error:
            return None, error

        response, error, kwargs = self.run_proto_request(proto=proto, **kwargs)
        if error:
            return None, error

        result, error = self._post_run_request(operation=operation, response=response)
        return result, error

    async def run_request_async(
        self, operation: Operation, **kwargs
    ) -> Tuple[Any, Any]:
        proto, error, kwargs = self._pre_run_request(operation=operation, **kwargs)
        if error:
            return None, error

        response, error, kwargs = self.run_proto_request(proto=proto, **kwargs)
        if error:
            return None, error

        result, error = self._post_run_request(operation=operation, response=response)
        return result, error

    def run_proto_request(self, proto: ProtoHttpRequest, **kwargs):
        http_client = kwargs.pop("http_client", self.get_http_client())
        request = http_client.create_request(proto=proto)
        raw_response, error = http_client.send_request(request, **kwargs)
        if error:
            return None, error, kwargs
        response, error = http_client.handle_response(raw_response, **kwargs)
        return response, error, kwargs

    async def run_proto_request_async(self, proto: ProtoHttpRequest, **kwargs):
        http_client = kwargs.pop("http_client", self.get_http_client())
        request = http_client.create_request(proto=proto)
        raw_response, error = await http_client.send_request_async(request, **kwargs)
        if error:
            return None, error
        response, error = http_client.handle_response(raw_response, **kwargs)
        return response, error, kwargs

    def _create_request_from_operation(
        self,
        operation: Operation,
        **kwargs,
    ):
        config_repo = kwargs.pop("config_repo", self.get_config_repository())
        token_repo = kwargs.pop("token_repo", self.get_token_repository())
        base_url = kwargs.pop("base_url", "") or config_repo.get_base_url()
        additional_headers = kwargs.pop("additional_headers", None)
        additional_headers_override = kwargs.pop("additional_headers_override", True)
        proto, error = create_proto_from_operation(
            operation=operation,
            base_url=base_url,
            config_repo=config_repo,
            token_repo=token_repo,
            additional_headers=additional_headers,
            additional_headers_override=additional_headers_override,
            **kwargs,
        )
        return proto, error, kwargs

    def _pre_run_request(self, operation: Operation, **kwargs):
        if operation.has_redirects() and "allow_redirects" not in kwargs:
            kwargs["allow_redirects"] = False

        for k, v in self.operation_preprocessors.items():
            operation, error = v(operation=operation, sdk=self)
            if error:
                return None, error, kwargs

        proto, error, kwargs = self._create_request_from_operation(
            operation=operation, **kwargs
        )
        if error:
            return None, error, kwargs

        for k, v in self.request_preprocessors.items():
            proto, error = v(proto=proto, sdk=self)
            if error:
                return None, error, kwargs

        return proto, None, kwargs

    def _post_run_request(self, operation: Operation, response: Any) -> Tuple[Any, Any]:
        response, error = operation.parse_response(*response)
        if error:
            return None, error

        for k, v in self.response_preprocessors.items():
            response, error = v(response=response, operation=operation, sdk=self)
            if error:
                return None, error

        return response, None

    # endregion Requests

    # region Utils

    @staticmethod
    def __clear_logger_handlers(logger_names: Iterable[str]) -> None:
        for logger_name in logger_names:
            logger = logging.getLogger(logger_name)
            logger.handlers.clear()

    @staticmethod
    def __is_valid_options_params(params: Any) -> Tuple[bool, list, dict]:
        invalid_result = False, [], {}
        if params is None:
            return True, [], {}
        if not isinstance(params, tuple):
            return invalid_result
        if len(params) != 2:
            return invalid_result
        args, kwargs = params
        if not isinstance(args, list):
            return invalid_result
        if not isinstance(kwargs, dict):
            return invalid_result
        return True, args, kwargs

    # endregion Utils


# region backwards compatibility


SDK = AccelByteSDK()


def s_is_initialized(sdk: Optional[AccelByteSDK] = None):
    sdk = sdk if sdk is not None else SDK
    return sdk.is_initialized()


def s_initialize(options=None, sdk: Optional[AccelByteSDK] = None):
    sdk = sdk if sdk is not None else SDK
    return sdk.initialize(options=options)


def s_reset(sdk: Optional[AccelByteSDK] = None):
    sdk = sdk if sdk is not None else SDK
    return sdk.reset()


def s_get_config_repository(raise_when_none=True, sdk: Optional[AccelByteSDK] = None):
    sdk = sdk if sdk is not None else SDK
    return sdk.get_config_repository(raise_when_none)


def s_set_config_repository(config_repository, sdk: Optional[AccelByteSDK] = None):
    sdk = sdk if sdk is not None else SDK
    return sdk.set_config_repository(config_repository)


def s_get_token_repository(raise_when_none=True, sdk: Optional[AccelByteSDK] = None):
    sdk = sdk if sdk is not None else SDK
    return sdk.get_token_repository(raise_when_none)


def s_set_token_repository(token_repository, sdk: Optional[AccelByteSDK] = None):
    sdk = sdk if sdk is not None else SDK
    return sdk.set_token_repository(token_repository)


def s_get_http_client(raise_when_none=True, sdk: Optional[AccelByteSDK] = None):
    sdk = sdk if sdk is not None else SDK
    return sdk.get_http_client(raise_when_none)


def s_set_http_client(http_client, sdk: Optional[AccelByteSDK] = None):
    sdk = sdk if sdk is not None else SDK
    return sdk.set_http_client(http_client)


def s_get_app_name(sdk: Optional[AccelByteSDK] = None):
    sdk = sdk if sdk is not None else SDK
    return sdk.get_app_name()


def s_get_app_version(sdk: Optional[AccelByteSDK] = None):
    sdk = sdk if sdk is not None else SDK
    return sdk.get_app_version()


def s_get_base_url(sdk: Optional[AccelByteSDK] = None):
    sdk = sdk if sdk is not None else SDK
    return sdk.get_base_url()


def s_get_client_auth(sdk: Optional[AccelByteSDK] = None):
    sdk = sdk if sdk is not None else SDK
    return sdk.get_client_auth()


def s_get_client_id(sdk: Optional[AccelByteSDK] = None):
    sdk = sdk if sdk is not None else SDK
    return sdk.get_client_id()


def s_get_client_secret(sdk: Optional[AccelByteSDK] = None):
    sdk = sdk if sdk is not None else SDK
    return sdk.get_client_secret()


def s_get_namespace(sdk: Optional[AccelByteSDK] = None):
    sdk = sdk if sdk is not None else SDK
    return sdk.get_namespace()


def s_get_access_token(sdk: Optional[AccelByteSDK] = None):
    sdk = sdk if sdk is not None else SDK
    return sdk.get_access_token()


def s_remove_token(sdk: Optional[AccelByteSDK] = None):
    sdk = sdk if sdk is not None else SDK
    return sdk.remove_token()


def s_set_token(token, sdk: Optional[AccelByteSDK] = None):
    sdk = sdk if sdk is not None else SDK
    return sdk.set_token(token)


def s_run_request(operation, sdk: Optional[AccelByteSDK] = None, **kwargs):
    sdk = sdk if sdk is not None else SDK
    return sdk.run_request(operation, **kwargs)


async def s_run_request_async(operation, sdk: Optional[AccelByteSDK] = None, **kwargs):
    sdk = sdk if sdk is not None else SDK
    return await sdk.run_request_async(operation, **kwargs)


def s_run_proto_request(proto, sdk: Optional[AccelByteSDK] = None, **kwargs):
    sdk = sdk if sdk is not None else SDK
    return sdk.run_proto_request(proto, **kwargs)


async def s_run_proto_request_async(
    proto, sdk: Optional[AccelByteSDK] = None, **kwargs
):
    sdk = sdk if sdk is not None else SDK
    return await sdk.run_proto_request_async(proto, **kwargs)


# endregion backwards compatibility



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: accelbyte_cloud_py_codegen

# AccelByte Cloud Platform Service (4.18.1)

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model
from ....core import StrEnum


class PaymentProviderEnum(StrEnum):
    ADYEN = "ADYEN"
    ALIPAY = "ALIPAY"
    CHECKOUT = "CHECKOUT"
    PAYPAL = "PAYPAL"
    STRIPE = "STRIPE"
    WALLET = "WALLET"
    WXPAY = "WXPAY"
    XSOLLA = "XSOLLA"


class PaymentUrlCreate(Model):
    """Payment url create (PaymentUrlCreate)

    Properties:
        payment_order_no: (paymentOrderNo) REQUIRED str

        payment_provider: (paymentProvider) REQUIRED Union[str, PaymentProviderEnum]

        return_url: (returnUrl) OPTIONAL str

        ui: (ui) OPTIONAL str

        zip_code: (zipCode) OPTIONAL str
    """

    # region fields

    payment_order_no: str  # REQUIRED
    payment_provider: Union[str, PaymentProviderEnum]  # REQUIRED
    return_url: str  # OPTIONAL
    ui: str  # OPTIONAL
    zip_code: str  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_payment_order_no(self, value: str) -> PaymentUrlCreate:
        self.payment_order_no = value
        return self

    def with_payment_provider(
        self, value: Union[str, PaymentProviderEnum]
    ) -> PaymentUrlCreate:
        self.payment_provider = value
        return self

    def with_return_url(self, value: str) -> PaymentUrlCreate:
        self.return_url = value
        return self

    def with_ui(self, value: str) -> PaymentUrlCreate:
        self.ui = value
        return self

    def with_zip_code(self, value: str) -> PaymentUrlCreate:
        self.zip_code = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "payment_order_no"):
            result["paymentOrderNo"] = str(self.payment_order_no)
        elif include_empty:
            result["paymentOrderNo"] = ""
        if hasattr(self, "payment_provider"):
            result["paymentProvider"] = str(self.payment_provider)
        elif include_empty:
            result["paymentProvider"] = Union[str, PaymentProviderEnum]()
        if hasattr(self, "return_url"):
            result["returnUrl"] = str(self.return_url)
        elif include_empty:
            result["returnUrl"] = ""
        if hasattr(self, "ui"):
            result["ui"] = str(self.ui)
        elif include_empty:
            result["ui"] = ""
        if hasattr(self, "zip_code"):
            result["zipCode"] = str(self.zip_code)
        elif include_empty:
            result["zipCode"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        payment_order_no: str,
        payment_provider: Union[str, PaymentProviderEnum],
        return_url: Optional[str] = None,
        ui: Optional[str] = None,
        zip_code: Optional[str] = None,
    ) -> PaymentUrlCreate:
        instance = cls()
        instance.payment_order_no = payment_order_no
        instance.payment_provider = payment_provider
        if return_url is not None:
            instance.return_url = return_url
        if ui is not None:
            instance.ui = ui
        if zip_code is not None:
            instance.zip_code = zip_code
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> PaymentUrlCreate:
        instance = cls()
        if not dict_:
            return instance
        if "paymentOrderNo" in dict_ and dict_["paymentOrderNo"] is not None:
            instance.payment_order_no = str(dict_["paymentOrderNo"])
        elif include_empty:
            instance.payment_order_no = ""
        if "paymentProvider" in dict_ and dict_["paymentProvider"] is not None:
            instance.payment_provider = str(dict_["paymentProvider"])
        elif include_empty:
            instance.payment_provider = Union[str, PaymentProviderEnum]()
        if "returnUrl" in dict_ and dict_["returnUrl"] is not None:
            instance.return_url = str(dict_["returnUrl"])
        elif include_empty:
            instance.return_url = ""
        if "ui" in dict_ and dict_["ui"] is not None:
            instance.ui = str(dict_["ui"])
        elif include_empty:
            instance.ui = ""
        if "zipCode" in dict_ and dict_["zipCode"] is not None:
            instance.zip_code = str(dict_["zipCode"])
        elif include_empty:
            instance.zip_code = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, PaymentUrlCreate]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[PaymentUrlCreate]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[PaymentUrlCreate, List[PaymentUrlCreate], Dict[Any, PaymentUrlCreate]]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "paymentOrderNo": "payment_order_no",
            "paymentProvider": "payment_provider",
            "returnUrl": "return_url",
            "ui": "ui",
            "zipCode": "zip_code",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "paymentOrderNo": True,
            "paymentProvider": True,
            "returnUrl": False,
            "ui": False,
            "zipCode": False,
        }

    @staticmethod
    def get_enum_map() -> Dict[str, List[Any]]:
        return {
            "paymentProvider": [
                "ADYEN",
                "ALIPAY",
                "CHECKOUT",
                "PAYPAL",
                "STRIPE",
                "WALLET",
                "WXPAY",
                "XSOLLA",
            ],
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: accelbyte_cloud_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

# AccelByte Cloud Basic Service (2.3.3)

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from .....core import Operation
from .....core import HeaderStr
from .....core import HttpResponse

from ...models import ErrorEntity
from ...models import FileUploadUrlInfo
from ...models import ValidationErrorEntity


class PublicGeneratedUserUploadContentUrl(Operation):
    """Generate Upload URL For User Content (publicGeneratedUserUploadContentUrl)

    Generate an upload URL for user content. It's valid for 10 minutes.
    There are 2 kinds of storage limitation per user : maximum file count and maximum file size.
    The threshold of those limitations is different between upload category that is used.
    Other detail info:

      * Required permission : resource = "NAMESPACE:{namespace}:USER:{userId}:FILEUPLOAD" , action=1 (CREATE)
      *  Action code : 11102
      *  Default maximum file count per user : 10 files
      *  Default maximum file size per user : 104857600 bytes
      *  Returns : URL data

    Required Permission(s):
        - NAMESPACE:{namespace}:USER:{userId}:FILEUPLOAD [CREATE]

    Properties:
        url: /basic/v1/public/namespaces/{namespace}/users/{userId}/files

        method: POST

        tags: ["FileUpload"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH] or [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

        category: (category) OPTIONAL str in query

        file_type: (fileType) REQUIRED str in query

    Responses:
        200: OK - FileUploadUrlInfo (Successful operation)

        400: Bad Request - ValidationErrorEntity (20002: validation error | 11121: Unable to {action}: category {category} is not valid | 11131: Unable to {action}: File type is not supported)

        401: Unauthorized - ErrorEntity (20001: unauthorized)

        403: Forbidden - ErrorEntity (20013: insufficient permission)

        409: Conflict - ErrorEntity (11132: Unable to {action}: file storage exceed limitation, user ID: {userId}, namespace: {namespace})

        500: Internal Server Error - ErrorEntity (20000: internal server error)
    """

    # region fields

    _url: str = "/basic/v1/public/namespaces/{namespace}/users/{userId}/files"
    _method: str = "POST"
    _consumes: List[str] = []
    _produces: List[str] = ["application/json"]
    _securities: List[List[str]] = [["BEARER_AUTH"], ["BEARER_AUTH"]]
    _location_query: str = None

    namespace: str  # REQUIRED in [path]
    user_id: str  # REQUIRED in [path]
    category: str  # OPTIONAL in [query]
    file_type: str  # REQUIRED in [query]

    # endregion fields

    # region properties

    @property
    def url(self) -> str:
        return self._url

    @property
    def method(self) -> str:
        return self._method

    @property
    def consumes(self) -> List[str]:
        return self._consumes

    @property
    def produces(self) -> List[str]:
        return self._produces

    @property
    def securities(self) -> List[List[str]]:
        return self._securities

    @property
    def location_query(self) -> str:
        return self._location_query

    # endregion properties

    # region get methods

    # endregion get methods

    # region get_x_params methods

    def get_all_params(self) -> dict:
        return {
            "path": self.get_path_params(),
            "query": self.get_query_params(),
        }

    def get_path_params(self) -> dict:
        result = {}
        if hasattr(self, "namespace"):
            result["namespace"] = self.namespace
        if hasattr(self, "user_id"):
            result["userId"] = self.user_id
        return result

    def get_query_params(self) -> dict:
        result = {}
        if hasattr(self, "category"):
            result["category"] = self.category
        if hasattr(self, "file_type"):
            result["fileType"] = self.file_type
        return result

    # endregion get_x_params methods

    # region is/has methods

    # endregion is/has methods

    # region with_x methods

    def with_namespace(self, value: str) -> PublicGeneratedUserUploadContentUrl:
        self.namespace = value
        return self

    def with_user_id(self, value: str) -> PublicGeneratedUserUploadContentUrl:
        self.user_id = value
        return self

    def with_category(self, value: str) -> PublicGeneratedUserUploadContentUrl:
        self.category = value
        return self

    def with_file_type(self, value: str) -> PublicGeneratedUserUploadContentUrl:
        self.file_type = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "namespace") and self.namespace:
            result["namespace"] = str(self.namespace)
        elif include_empty:
            result["namespace"] = ""
        if hasattr(self, "user_id") and self.user_id:
            result["userId"] = str(self.user_id)
        elif include_empty:
            result["userId"] = ""
        if hasattr(self, "category") and self.category:
            result["category"] = str(self.category)
        elif include_empty:
            result["category"] = ""
        if hasattr(self, "file_type") and self.file_type:
            result["fileType"] = str(self.file_type)
        elif include_empty:
            result["fileType"] = ""
        return result

    # endregion to methods

    # region response methods

    # noinspection PyMethodMayBeStatic
    def parse_response(
        self, code: int, content_type: str, content: Any
    ) -> Tuple[
        Union[None, FileUploadUrlInfo],
        Union[None, ErrorEntity, HttpResponse, ValidationErrorEntity],
    ]:
        """Parse the given response.

        200: OK - FileUploadUrlInfo (Successful operation)

        400: Bad Request - ValidationErrorEntity (20002: validation error | 11121: Unable to {action}: category {category} is not valid | 11131: Unable to {action}: File type is not supported)

        401: Unauthorized - ErrorEntity (20001: unauthorized)

        403: Forbidden - ErrorEntity (20013: insufficient permission)

        409: Conflict - ErrorEntity (11132: Unable to {action}: file storage exceed limitation, user ID: {userId}, namespace: {namespace})

        500: Internal Server Error - ErrorEntity (20000: internal server error)

        ---: HttpResponse (Undocumented Response)

        ---: HttpResponse (Unexpected Content-Type Error)

        ---: HttpResponse (Unhandled Error)
        """
        pre_processed_response, error = self.pre_process_response(
            code=code, content_type=content_type, content=content
        )
        if error is not None:
            return None, None if error.is_no_content() else error
        code, content_type, content = pre_processed_response

        if code == 200:
            return FileUploadUrlInfo.create_from_dict(content), None
        if code == 400:
            return None, ValidationErrorEntity.create_from_dict(content)
        if code == 401:
            return None, ErrorEntity.create_from_dict(content)
        if code == 403:
            return None, ErrorEntity.create_from_dict(content)
        if code == 409:
            return None, ErrorEntity.create_from_dict(content)
        if code == 500:
            return None, ErrorEntity.create_from_dict(content)

        return self.handle_undocumented_response(
            code=code, content_type=content_type, content=content
        )

    # endregion response methods

    # region static methods

    @classmethod
    def create(
        cls,
        namespace: str,
        user_id: str,
        file_type: str,
        category: Optional[str] = None,
    ) -> PublicGeneratedUserUploadContentUrl:
        instance = cls()
        instance.namespace = namespace
        instance.user_id = user_id
        instance.file_type = file_type
        if category is not None:
            instance.category = category
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> PublicGeneratedUserUploadContentUrl:
        instance = cls()
        if "namespace" in dict_ and dict_["namespace"] is not None:
            instance.namespace = str(dict_["namespace"])
        elif include_empty:
            instance.namespace = ""
        if "userId" in dict_ and dict_["userId"] is not None:
            instance.user_id = str(dict_["userId"])
        elif include_empty:
            instance.user_id = ""
        if "category" in dict_ and dict_["category"] is not None:
            instance.category = str(dict_["category"])
        elif include_empty:
            instance.category = ""
        if "fileType" in dict_ and dict_["fileType"] is not None:
            instance.file_type = str(dict_["fileType"])
        elif include_empty:
            instance.file_type = ""
        return instance

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "namespace": "namespace",
            "userId": "user_id",
            "category": "category",
            "fileType": "file_type",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "namespace": True,
            "userId": True,
            "category": False,
            "fileType": True,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: accelbyte_cloud_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import HeaderStr
from ....core import get_namespace as get_services_namespace
from ....core import run_request
from ....core import run_request_async
from ....core import same_doc_as

from ..models import ModelsChannelRequest
from ..models import ModelsChannelResponse
from ..models import ModelsPaginatedGetChannelResponse
from ..models import ResponseError

from ..operations.public_channel import CreateChannel
from ..operations.public_channel import DeleteChannel
from ..operations.public_channel import GetChannels
from ..operations.public_channel import UpdateChannel


@same_doc_as(CreateChannel)
def create_channel(
    body: ModelsChannelRequest,
    user_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = CreateChannel.create(
        body=body,
        user_id=user_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(CreateChannel)
async def create_channel_async(
    body: ModelsChannelRequest,
    user_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = CreateChannel.create(
        body=body,
        user_id=user_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(DeleteChannel)
def delete_channel(
    channel_id: str,
    user_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = DeleteChannel.create(
        channel_id=channel_id,
        user_id=user_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(DeleteChannel)
async def delete_channel_async(
    channel_id: str,
    user_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = DeleteChannel.create(
        channel_id=channel_id,
        user_id=user_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(GetChannels)
def get_channels(
    user_id: str,
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetChannels.create(
        user_id=user_id,
        limit=limit,
        offset=offset,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(GetChannels)
async def get_channels_async(
    user_id: str,
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetChannels.create(
        user_id=user_id,
        limit=limit,
        offset=offset,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(UpdateChannel)
def update_channel(
    body: ModelsChannelRequest,
    channel_id: str,
    user_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = UpdateChannel.create(
        body=body,
        channel_id=channel_id,
        user_id=user_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(UpdateChannel)
async def update_channel_async(
    body: ModelsChannelRequest,
    channel_id: str,
    user_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = UpdateChannel.create(
        body=body,
        channel_id=channel_id,
        user_id=user_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: accelbyte_cloud_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

# AccelByte Cloud Platform Service (4.18.1)

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from .....core import Operation
from .....core import HeaderStr
from .....core import HttpResponse
from .....core import StrEnum

from ...models import ErrorEntity
from ...models import FullItemPagingSlicedResult


class ItemTypeEnum(StrEnum):
    APP = "APP"
    BUNDLE = "BUNDLE"
    CODE = "CODE"
    COINS = "COINS"
    EXTENSION = "EXTENSION"
    INGAMEITEM = "INGAMEITEM"
    LOOTBOX = "LOOTBOX"
    MEDIA = "MEDIA"
    OPTIONBOX = "OPTIONBOX"
    SEASON = "SEASON"
    SUBSCRIPTION = "SUBSCRIPTION"


class SearchItems(Operation):
    """Search items by keyword (searchItems)

    This API is used to search items by keyword in title, description and long description within a store.

    Other detail info:

      * Required permission : resource="ADMIN:NAMESPACE:{namespace}:ITEM", action=2 (READ)
      *  Returns : the list of items

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:ITEM [READ]

    Properties:
        url: /platform/admin/namespaces/{namespace}/items/search

        method: GET

        tags: ["Item"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH] or [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        active_only: (activeOnly) OPTIONAL bool in query

        item_type: (itemType) OPTIONAL Union[str, ItemTypeEnum] in query

        limit: (limit) OPTIONAL int in query

        offset: (offset) OPTIONAL int in query

        sort_by: (sortBy) OPTIONAL str in query

        store_id: (storeId) OPTIONAL str in query

        keyword: (keyword) REQUIRED str in query

        language: (language) REQUIRED str in query

    Responses:
        200: OK - FullItemPagingSlicedResult (successful operation)

        404: Not Found - ErrorEntity (30141: Store [{storeId}] does not exist in namespace [{namespace}] | 30142: Published store does not exist in namespace [{namespace}])
    """

    # region fields

    _url: str = "/platform/admin/namespaces/{namespace}/items/search"
    _method: str = "GET"
    _consumes: List[str] = []
    _produces: List[str] = ["application/json"]
    _securities: List[List[str]] = [["BEARER_AUTH"], ["BEARER_AUTH"]]
    _location_query: str = None

    namespace: str  # REQUIRED in [path]
    active_only: bool  # OPTIONAL in [query]
    item_type: Union[str, ItemTypeEnum]  # OPTIONAL in [query]
    limit: int  # OPTIONAL in [query]
    offset: int  # OPTIONAL in [query]
    sort_by: str  # OPTIONAL in [query]
    store_id: str  # OPTIONAL in [query]
    keyword: str  # REQUIRED in [query]
    language: str  # REQUIRED in [query]

    # endregion fields

    # region properties

    @property
    def url(self) -> str:
        return self._url

    @property
    def method(self) -> str:
        return self._method

    @property
    def consumes(self) -> List[str]:
        return self._consumes

    @property
    def produces(self) -> List[str]:
        return self._produces

    @property
    def securities(self) -> List[List[str]]:
        return self._securities

    @property
    def location_query(self) -> str:
        return self._location_query

    # endregion properties

    # region get methods

    # endregion get methods

    # region get_x_params methods

    def get_all_params(self) -> dict:
        return {
            "path": self.get_path_params(),
            "query": self.get_query_params(),
        }

    def get_path_params(self) -> dict:
        result = {}
        if hasattr(self, "namespace"):
            result["namespace"] = self.namespace
        return result

    def get_query_params(self) -> dict:
        result = {}
        if hasattr(self, "active_only"):
            result["activeOnly"] = self.active_only
        if hasattr(self, "item_type"):
            result["itemType"] = self.item_type
        if hasattr(self, "limit"):
            result["limit"] = self.limit
        if hasattr(self, "offset"):
            result["offset"] = self.offset
        if hasattr(self, "sort_by"):
            result["sortBy"] = self.sort_by
        if hasattr(self, "store_id"):
            result["storeId"] = self.store_id
        if hasattr(self, "keyword"):
            result["keyword"] = self.keyword
        if hasattr(self, "language"):
            result["language"] = self.language
        return result

    # endregion get_x_params methods

    # region is/has methods

    # endregion is/has methods

    # region with_x methods

    def with_namespace(self, value: str) -> SearchItems:
        self.namespace = value
        return self

    def with_active_only(self, value: bool) -> SearchItems:
        self.active_only = value
        return self

    def with_item_type(self, value: Union[str, ItemTypeEnum]) -> SearchItems:
        self.item_type = value
        return self

    def with_limit(self, value: int) -> SearchItems:
        self.limit = value
        return self

    def with_offset(self, value: int) -> SearchItems:
        self.offset = value
        return self

    def with_sort_by(self, value: str) -> SearchItems:
        self.sort_by = value
        return self

    def with_store_id(self, value: str) -> SearchItems:
        self.store_id = value
        return self

    def with_keyword(self, value: str) -> SearchItems:
        self.keyword = value
        return self

    def with_language(self, value: str) -> SearchItems:
        self.language = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "namespace") and self.namespace:
            result["namespace"] = str(self.namespace)
        elif include_empty:
            result["namespace"] = ""
        if hasattr(self, "active_only") and self.active_only:
            result["activeOnly"] = bool(self.active_only)
        elif include_empty:
            result["activeOnly"] = False
        if hasattr(self, "item_type") and self.item_type:
            result["itemType"] = str(self.item_type)
        elif include_empty:
            result["itemType"] = Union[str, ItemTypeEnum]()
        if hasattr(self, "limit") and self.limit:
            result["limit"] = int(self.limit)
        elif include_empty:
            result["limit"] = 0
        if hasattr(self, "offset") and self.offset:
            result["offset"] = int(self.offset)
        elif include_empty:
            result["offset"] = 0
        if hasattr(self, "sort_by") and self.sort_by:
            result["sortBy"] = str(self.sort_by)
        elif include_empty:
            result["sortBy"] = ""
        if hasattr(self, "store_id") and self.store_id:
            result["storeId"] = str(self.store_id)
        elif include_empty:
            result["storeId"] = ""
        if hasattr(self, "keyword") and self.keyword:
            result["keyword"] = str(self.keyword)
        elif include_empty:
            result["keyword"] = ""
        if hasattr(self, "language") and self.language:
            result["language"] = str(self.language)
        elif include_empty:
            result["language"] = ""
        return result

    # endregion to methods

    # region response methods

    # noinspection PyMethodMayBeStatic
    def parse_response(
        self, code: int, content_type: str, content: Any
    ) -> Tuple[
        Union[None, FullItemPagingSlicedResult], Union[None, ErrorEntity, HttpResponse]
    ]:
        """Parse the given response.

        200: OK - FullItemPagingSlicedResult (successful operation)

        404: Not Found - ErrorEntity (30141: Store [{storeId}] does not exist in namespace [{namespace}] | 30142: Published store does not exist in namespace [{namespace}])

        ---: HttpResponse (Undocumented Response)

        ---: HttpResponse (Unexpected Content-Type Error)

        ---: HttpResponse (Unhandled Error)
        """
        pre_processed_response, error = self.pre_process_response(
            code=code, content_type=content_type, content=content
        )
        if error is not None:
            return None, None if error.is_no_content() else error
        code, content_type, content = pre_processed_response

        if code == 200:
            return FullItemPagingSlicedResult.create_from_dict(content), None
        if code == 404:
            return None, ErrorEntity.create_from_dict(content)

        return self.handle_undocumented_response(
            code=code, content_type=content_type, content=content
        )

    # endregion response methods

    # region static methods

    @classmethod
    def create(
        cls,
        namespace: str,
        keyword: str,
        language: str,
        active_only: Optional[bool] = None,
        item_type: Optional[Union[str, ItemTypeEnum]] = None,
        limit: Optional[int] = None,
        offset: Optional[int] = None,
        sort_by: Optional[str] = None,
        store_id: Optional[str] = None,
    ) -> SearchItems:
        instance = cls()
        instance.namespace = namespace
        instance.keyword = keyword
        instance.language = language
        if active_only is not None:
            instance.active_only = active_only
        if item_type is not None:
            instance.item_type = item_type
        if limit is not None:
            instance.limit = limit
        if offset is not None:
            instance.offset = offset
        if sort_by is not None:
            instance.sort_by = sort_by
        if store_id is not None:
            instance.store_id = store_id
        return instance

    @classmethod
    def create_from_dict(cls, dict_: dict, include_empty: bool = False) -> SearchItems:
        instance = cls()
        if "namespace" in dict_ and dict_["namespace"] is not None:
            instance.namespace = str(dict_["namespace"])
        elif include_empty:
            instance.namespace = ""
        if "activeOnly" in dict_ and dict_["activeOnly"] is not None:
            instance.active_only = bool(dict_["activeOnly"])
        elif include_empty:
            instance.active_only = False
        if "itemType" in dict_ and dict_["itemType"] is not None:
            instance.item_type = str(dict_["itemType"])
        elif include_empty:
            instance.item_type = Union[str, ItemTypeEnum]()
        if "limit" in dict_ and dict_["limit"] is not None:
            instance.limit = int(dict_["limit"])
        elif include_empty:
            instance.limit = 0
        if "offset" in dict_ and dict_["offset"] is not None:
            instance.offset = int(dict_["offset"])
        elif include_empty:
            instance.offset = 0
        if "sortBy" in dict_ and dict_["sortBy"] is not None:
            instance.sort_by = str(dict_["sortBy"])
        elif include_empty:
            instance.sort_by = ""
        if "storeId" in dict_ and dict_["storeId"] is not None:
            instance.store_id = str(dict_["storeId"])
        elif include_empty:
            instance.store_id = ""
        if "keyword" in dict_ and dict_["keyword"] is not None:
            instance.keyword = str(dict_["keyword"])
        elif include_empty:
            instance.keyword = ""
        if "language" in dict_ and dict_["language"] is not None:
            instance.language = str(dict_["language"])
        elif include_empty:
            instance.language = ""
        return instance

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "namespace": "namespace",
            "activeOnly": "active_only",
            "itemType": "item_type",
            "limit": "limit",
            "offset": "offset",
            "sortBy": "sort_by",
            "storeId": "store_id",
            "keyword": "keyword",
            "language": "language",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "namespace": True,
            "activeOnly": False,
            "itemType": False,
            "limit": False,
            "offset": False,
            "sortBy": False,
            "storeId": False,
            "keyword": True,
            "language": True,
        }

    @staticmethod
    def get_enum_map() -> Dict[str, List[Any]]:
        return {
            "itemType": [
                "APP",
                "BUNDLE",
                "CODE",
                "COINS",
                "EXTENSION",
                "INGAMEITEM",
                "LOOTBOX",
                "MEDIA",
                "OPTIONBOX",
                "SEASON",
                "SUBSCRIPTION",
            ],  # in query
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: accelbyte_cloud_py_codegen

# AccelByte Cloud Legal Service (1.25.1)

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model


class AcceptAgreementRequest(Model):
    """Accept agreement request (AcceptAgreementRequest)

    Properties:
        is_accepted: (isAccepted) REQUIRED bool

        localized_policy_version_id: (localizedPolicyVersionId) REQUIRED str

        policy_id: (policyId) REQUIRED str

        policy_version_id: (policyVersionId) REQUIRED str

        is_need_to_send_event_marketing: (isNeedToSendEventMarketing) OPTIONAL bool
    """

    # region fields

    is_accepted: bool  # REQUIRED
    localized_policy_version_id: str  # REQUIRED
    policy_id: str  # REQUIRED
    policy_version_id: str  # REQUIRED
    is_need_to_send_event_marketing: bool  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_is_accepted(self, value: bool) -> AcceptAgreementRequest:
        self.is_accepted = value
        return self

    def with_localized_policy_version_id(self, value: str) -> AcceptAgreementRequest:
        self.localized_policy_version_id = value
        return self

    def with_policy_id(self, value: str) -> AcceptAgreementRequest:
        self.policy_id = value
        return self

    def with_policy_version_id(self, value: str) -> AcceptAgreementRequest:
        self.policy_version_id = value
        return self

    def with_is_need_to_send_event_marketing(
        self, value: bool
    ) -> AcceptAgreementRequest:
        self.is_need_to_send_event_marketing = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "is_accepted"):
            result["isAccepted"] = bool(self.is_accepted)
        elif include_empty:
            result["isAccepted"] = False
        if hasattr(self, "localized_policy_version_id"):
            result["localizedPolicyVersionId"] = str(self.localized_policy_version_id)
        elif include_empty:
            result["localizedPolicyVersionId"] = ""
        if hasattr(self, "policy_id"):
            result["policyId"] = str(self.policy_id)
        elif include_empty:
            result["policyId"] = ""
        if hasattr(self, "policy_version_id"):
            result["policyVersionId"] = str(self.policy_version_id)
        elif include_empty:
            result["policyVersionId"] = ""
        if hasattr(self, "is_need_to_send_event_marketing"):
            result["isNeedToSendEventMarketing"] = bool(
                self.is_need_to_send_event_marketing
            )
        elif include_empty:
            result["isNeedToSendEventMarketing"] = False
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        is_accepted: bool,
        localized_policy_version_id: str,
        policy_id: str,
        policy_version_id: str,
        is_need_to_send_event_marketing: Optional[bool] = None,
    ) -> AcceptAgreementRequest:
        instance = cls()
        instance.is_accepted = is_accepted
        instance.localized_policy_version_id = localized_policy_version_id
        instance.policy_id = policy_id
        instance.policy_version_id = policy_version_id
        if is_need_to_send_event_marketing is not None:
            instance.is_need_to_send_event_marketing = is_need_to_send_event_marketing
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> AcceptAgreementRequest:
        instance = cls()
        if not dict_:
            return instance
        if "isAccepted" in dict_ and dict_["isAccepted"] is not None:
            instance.is_accepted = bool(dict_["isAccepted"])
        elif include_empty:
            instance.is_accepted = False
        if (
            "localizedPolicyVersionId" in dict_
            and dict_["localizedPolicyVersionId"] is not None
        ):
            instance.localized_policy_version_id = str(
                dict_["localizedPolicyVersionId"]
            )
        elif include_empty:
            instance.localized_policy_version_id = ""
        if "policyId" in dict_ and dict_["policyId"] is not None:
            instance.policy_id = str(dict_["policyId"])
        elif include_empty:
            instance.policy_id = ""
        if "policyVersionId" in dict_ and dict_["policyVersionId"] is not None:
            instance.policy_version_id = str(dict_["policyVersionId"])
        elif include_empty:
            instance.policy_version_id = ""
        if (
            "isNeedToSendEventMarketing" in dict_
            and dict_["isNeedToSendEventMarketing"] is not None
        ):
            instance.is_need_to_send_event_marketing = bool(
                dict_["isNeedToSendEventMarketing"]
            )
        elif include_empty:
            instance.is_need_to_send_event_marketing = False
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, AcceptAgreementRequest]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[AcceptAgreementRequest]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        AcceptAgreementRequest,
        List[AcceptAgreementRequest],
        Dict[Any, AcceptAgreementRequest],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "isAccepted": "is_accepted",
            "localizedPolicyVersionId": "localized_policy_version_id",
            "policyId": "policy_id",
            "policyVersionId": "policy_version_id",
            "isNeedToSendEventMarketing": "is_need_to_send_event_marketing",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "isAccepted": True,
            "localizedPolicyVersionId": True,
            "policyId": True,
            "policyVersionId": True,
            "isNeedToSendEventMarketing": False,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: accelbyte_cloud_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import HeaderStr
from ....core import get_namespace as get_services_namespace
from ....core import run_request
from ....core import run_request_async
from ....core import same_doc_as

from ..models import ModelsDefaultProvider

from ..operations.public import GetDefaultProvider
from ..operations.public import ListProviders
from ..operations.public import ListProvidersByRegion


@same_doc_as(GetDefaultProvider)
def get_default_provider(
    x_additional_headers: Optional[Dict[str, str]] = None, **kwargs
):
    request = GetDefaultProvider.create()
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(GetDefaultProvider)
async def get_default_provider_async(
    x_additional_headers: Optional[Dict[str, str]] = None, **kwargs
):
    request = GetDefaultProvider.create()
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(ListProviders)
def list_providers(x_additional_headers: Optional[Dict[str, str]] = None, **kwargs):
    request = ListProviders.create()
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(ListProviders)
async def list_providers_async(
    x_additional_headers: Optional[Dict[str, str]] = None, **kwargs
):
    request = ListProviders.create()
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(ListProvidersByRegion)
def list_providers_by_region(
    region: str, x_additional_headers: Optional[Dict[str, str]] = None, **kwargs
):
    request = ListProvidersByRegion.create(
        region=region,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(ListProvidersByRegion)
async def list_providers_by_region_async(
    region: str, x_additional_headers: Optional[Dict[str, str]] = None, **kwargs
):
    request = ListProvidersByRegion.create(
        region=region,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: accelbyte_cloud_py_codegen

# AccelByte Cloud Social Service (1.31.1)

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model


class ADTOObjectForUserStatItemValue(Model):
    """A DTO object for user stat item value (A DTO object for user stat item value)

    Properties:
        profile_id: (profileId) OPTIONAL str

        stat_code: (statCode) OPTIONAL str

        value: (value) OPTIONAL float
    """

    # region fields

    profile_id: str  # OPTIONAL
    stat_code: str  # OPTIONAL
    value: float  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_profile_id(self, value: str) -> ADTOObjectForUserStatItemValue:
        self.profile_id = value
        return self

    def with_stat_code(self, value: str) -> ADTOObjectForUserStatItemValue:
        self.stat_code = value
        return self

    def with_value(self, value: float) -> ADTOObjectForUserStatItemValue:
        self.value = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "profile_id"):
            result["profileId"] = str(self.profile_id)
        elif include_empty:
            result["profileId"] = ""
        if hasattr(self, "stat_code"):
            result["statCode"] = str(self.stat_code)
        elif include_empty:
            result["statCode"] = ""
        if hasattr(self, "value"):
            result["value"] = float(self.value)
        elif include_empty:
            result["value"] = 0.0
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        profile_id: Optional[str] = None,
        stat_code: Optional[str] = None,
        value: Optional[float] = None,
    ) -> ADTOObjectForUserStatItemValue:
        instance = cls()
        if profile_id is not None:
            instance.profile_id = profile_id
        if stat_code is not None:
            instance.stat_code = stat_code
        if value is not None:
            instance.value = value
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ADTOObjectForUserStatItemValue:
        instance = cls()
        if not dict_:
            return instance
        if "profileId" in dict_ and dict_["profileId"] is not None:
            instance.profile_id = str(dict_["profileId"])
        elif include_empty:
            instance.profile_id = ""
        if "statCode" in dict_ and dict_["statCode"] is not None:
            instance.stat_code = str(dict_["statCode"])
        elif include_empty:
            instance.stat_code = ""
        if "value" in dict_ and dict_["value"] is not None:
            instance.value = float(dict_["value"])
        elif include_empty:
            instance.value = 0.0
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ADTOObjectForUserStatItemValue]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ADTOObjectForUserStatItemValue]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        ADTOObjectForUserStatItemValue,
        List[ADTOObjectForUserStatItemValue],
        Dict[Any, ADTOObjectForUserStatItemValue],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "profileId": "profile_id",
            "statCode": "stat_code",
            "value": "value",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "profileId": False,
            "statCode": False,
            "value": False,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: accelbyte_cloud_py_codegen

"""Auto-generated package that contains models used by the AccelByte Cloud Session Service."""

__version__ = "2.3.0"
__author__ = "AccelByte"
__email__ = "dev@accelbyte.net"

# pylint: disable=line-too-long

from .apimodels_configuration_template_response import (
    ApimodelsConfigurationTemplateResponse,
)
from .apimodels_configuration_templates_response import (
    ApimodelsConfigurationTemplatesResponse,
)
from .apimodels_create_configuration_template_request import (
    ApimodelsCreateConfigurationTemplateRequest,
)
from .apimodels_create_game_session_request import ApimodelsCreateGameSessionRequest
from .apimodels_create_party_request import ApimodelsCreatePartyRequest
from .apimodels_ds_information_response import ApimodelsDSInformationResponse
from .apimodels_game_session_query_response import ApimodelsGameSessionQueryResponse
from .apimodels_game_session_response import ApimodelsGameSessionResponse
from .apimodels_join_by_code_request import ApimodelsJoinByCodeRequest
from .apimodels_kick_response import ApimodelsKickResponse
from .apimodels_pagination import ApimodelsPagination
from .apimodels_party_query_response import ApimodelsPartyQueryResponse
from .apimodels_party_session_response import ApimodelsPartySessionResponse
from .apimodels_promote_leader_request import ApimodelsPromoteLeaderRequest
from .apimodels_public_configuration import ApimodelsPublicConfiguration
from .apimodels_request_member import ApimodelsRequestMember
from .apimodels_session_invite_request import ApimodelsSessionInviteRequest
from .apimodels_update_configuration_template_request import (
    ApimodelsUpdateConfigurationTemplateRequest,
)
from .apimodels_update_game_session_member_status_response import (
    ApimodelsUpdateGameSessionMemberStatusResponse,
)
from .apimodels_update_game_session_request import ApimodelsUpdateGameSessionRequest
from .apimodels_update_party_request import ApimodelsUpdatePartyRequest
from .apimodels_user_response import ApimodelsUserResponse
from .models_game_server import ModelsGameServer
from .models_team import ModelsTeam
from .response_error import ResponseError



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: accelbyte_cloud_py_codegen

# AccelByte Cloud Group Service (2.12.0)

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from .utils import randomize

from ..api.group.models import ModelsAssignRoleToMemberRequestV1
from ..api.group.models import ModelsCreateGroupConfigurationRequestV1
from ..api.group.models import ModelsCreateGroupConfigurationResponseV1
from ..api.group.models import ModelsCreateMemberRoleRequestV1
from ..api.group.models import ModelsCreateMemberRoleResponseV1
from ..api.group.models import ModelsGetGroupConfigurationResponseV1
from ..api.group.models import ModelsGetGroupMemberListResponseV1
from ..api.group.models import ModelsGetGroupsListResponseV1
from ..api.group.models import ModelsGetMemberRequestsListResponseV1
from ..api.group.models import ModelsGetMemberRoleResponseV1
from ..api.group.models import ModelsGetMemberRolesListResponseV1
from ..api.group.models import ModelsGetUserGroupInformationResponseV1
from ..api.group.models import ModelsGroupMember
from ..api.group.models import ModelsGroupResponseV1
from ..api.group.models import ModelsGroupRule
from ..api.group.models import ModelsGroupRuleGroupCustomRule
from ..api.group.models import ModelsJoinGroupResponseV1
from ..api.group.models import ModelsKickGroupMemberResponseV1
from ..api.group.models import ModelsLeaveGroupResponseV1
from ..api.group.models import ModelsListConfigurationResponseV1
from ..api.group.models import ModelsMemberRequestGroupResponseV1
from ..api.group.models import ModelsMemberRequestResponseV1
from ..api.group.models import ModelsPagination
from ..api.group.models import ModelsPublicCreateNewGroupRequestV1
from ..api.group.models import ModelsRemoveRoleFromMemberRequestV1
from ..api.group.models import ModelsRolePermission
from ..api.group.models import ModelsRule
from ..api.group.models import ModelsRuleInformation
from ..api.group.models import ModelsUpdateGroupConfigurationGlobalRulesRequestV1
from ..api.group.models import ModelsUpdateGroupConfigurationRequestV1
from ..api.group.models import ModelsUpdateGroupConfigurationResponseV1
from ..api.group.models import ModelsUpdateGroupCustomAttributesRequestV1
from ..api.group.models import ModelsUpdateGroupCustomRuleRequestV1
from ..api.group.models import ModelsUpdateGroupPredefinedRuleRequestV1
from ..api.group.models import ModelsUpdateGroupRequestV1
from ..api.group.models import ModelsUpdateGroupRequestV1CustomAttributes
from ..api.group.models import ModelsUpdateMemberRolePermissionsRequestV1
from ..api.group.models import ModelsUpdateMemberRoleRequestV1
from ..api.group.models import ModelsUpdateMemberRoleResponseV1
from ..api.group.models import ModelsUserInvitationResponseV1
from ..api.group.models import ResponseErrorResponse


def create_models_assign_role_to_member_request_v1_example() -> ModelsAssignRoleToMemberRequestV1:
    instance = ModelsAssignRoleToMemberRequestV1()
    instance.user_id = randomize("uid")
    return instance


def create_models_create_group_configuration_request_v1_example() -> ModelsCreateGroupConfigurationRequestV1:
    instance = ModelsCreateGroupConfigurationRequestV1()
    instance.configuration_code = randomize()
    instance.description = randomize()
    instance.global_rules = [create_models_rule_example()]
    instance.group_admin_role_id = randomize()
    instance.group_max_member = randomize("int", min_val=1, max_val=1000)
    instance.group_member_role_id = randomize()
    instance.name = randomize()
    return instance


def create_models_create_group_configuration_response_v1_example() -> ModelsCreateGroupConfigurationResponseV1:
    instance = ModelsCreateGroupConfigurationResponseV1()
    instance.configuration_code = randomize()
    instance.description = randomize()
    instance.global_rules = [create_models_rule_example()]
    instance.group_admin_role_id = randomize()
    instance.group_max_member = randomize("int", min_val=1, max_val=1000)
    instance.group_member_role_id = randomize()
    instance.name = randomize()
    return instance


def create_models_create_member_role_request_v1_example() -> ModelsCreateMemberRoleRequestV1:
    instance = ModelsCreateMemberRoleRequestV1()
    instance.member_role_name = randomize()
    instance.member_role_permissions = [create_models_role_permission_example()]
    return instance


def create_models_create_member_role_response_v1_example() -> ModelsCreateMemberRoleResponseV1:
    instance = ModelsCreateMemberRoleResponseV1()
    instance.member_role_id = randomize()
    instance.member_role_name = randomize()
    instance.member_role_permissions = [create_models_role_permission_example()]
    return instance


def create_models_get_group_configuration_response_v1_example() -> ModelsGetGroupConfigurationResponseV1:
    instance = ModelsGetGroupConfigurationResponseV1()
    instance.configuration_code = randomize()
    instance.description = randomize()
    instance.global_rules = [create_models_rule_example()]
    instance.group_admin_role_id = randomize()
    instance.group_max_member = randomize("int", min_val=1, max_val=1000)
    instance.group_member_role_id = randomize()
    instance.name = randomize()
    instance.namespace = randomize("slug")
    return instance


def create_models_get_group_member_list_response_v1_example() -> ModelsGetGroupMemberListResponseV1:
    instance = ModelsGetGroupMemberListResponseV1()
    instance.data = [create_models_get_user_group_information_response_v1_example()]
    instance.paging = create_models_pagination_example()
    return instance


def create_models_get_groups_list_response_v1_example() -> ModelsGetGroupsListResponseV1:
    instance = ModelsGetGroupsListResponseV1()
    instance.data = [create_models_group_response_v1_example()]
    instance.paging = create_models_pagination_example()
    return instance


def create_models_get_member_requests_list_response_v1_example() -> ModelsGetMemberRequestsListResponseV1:
    instance = ModelsGetMemberRequestsListResponseV1()
    instance.data = [create_models_member_request_response_v1_example()]
    instance.paging = create_models_pagination_example()
    return instance


def create_models_get_member_role_response_v1_example() -> ModelsGetMemberRoleResponseV1:
    instance = ModelsGetMemberRoleResponseV1()
    instance.member_role_id = randomize()
    instance.member_role_name = randomize()
    instance.member_role_permissions = [create_models_role_permission_example()]
    return instance


def create_models_get_member_roles_list_response_v1_example() -> ModelsGetMemberRolesListResponseV1:
    instance = ModelsGetMemberRolesListResponseV1()
    instance.data = [create_models_get_member_role_response_v1_example()]
    instance.paging = create_models_pagination_example()
    return instance


def create_models_get_user_group_information_response_v1_example() -> ModelsGetUserGroupInformationResponseV1:
    instance = ModelsGetUserGroupInformationResponseV1()
    instance.group_id = randomize()
    instance.member_role_id = [randomize()]
    instance.user_id = randomize("uid")
    instance.status = randomize()
    return instance


def create_models_group_member_example() -> ModelsGroupMember:
    instance = ModelsGroupMember()
    instance.member_role_id = [randomize()]
    instance.user_id = randomize("uid")
    return instance


def create_models_group_response_v1_example() -> ModelsGroupResponseV1:
    instance = ModelsGroupResponseV1()
    instance.configuration_code = randomize()
    instance.custom_attributes = {randomize(): randomize()}
    instance.group_description = randomize()
    instance.group_icon = randomize()
    instance.group_id = randomize()
    instance.group_max_member = randomize("int", min_val=1, max_val=1000)
    instance.group_members = [create_models_group_member_example()]
    instance.group_name = randomize()
    instance.group_region = randomize()
    instance.group_rules = create_models_group_rule_example()
    instance.group_type = randomize()
    return instance


def create_models_group_rule_example() -> ModelsGroupRule:
    instance = ModelsGroupRule()
    instance.group_custom_rule = create_models_group_rule_group_custom_rule_example()
    instance.group_predefined_rules = [create_models_rule_example()]
    return instance


def create_models_group_rule_group_custom_rule_example() -> ModelsGroupRuleGroupCustomRule:
    instance = ModelsGroupRuleGroupCustomRule()
    return instance


def create_models_join_group_response_v1_example() -> ModelsJoinGroupResponseV1:
    instance = ModelsJoinGroupResponseV1()
    instance.group_id = randomize()
    instance.status = randomize()
    instance.user_id = randomize("uid")
    return instance


def create_models_kick_group_member_response_v1_example() -> ModelsKickGroupMemberResponseV1:
    instance = ModelsKickGroupMemberResponseV1()
    instance.group_id = randomize()
    instance.kicked_user_id = randomize()
    return instance


def create_models_leave_group_response_v1_example() -> ModelsLeaveGroupResponseV1:
    instance = ModelsLeaveGroupResponseV1()
    instance.group_id = randomize()
    instance.user_id = randomize("uid")
    return instance


def create_models_list_configuration_response_v1_example() -> ModelsListConfigurationResponseV1:
    instance = ModelsListConfigurationResponseV1()
    instance.data = [create_models_get_group_configuration_response_v1_example()]
    instance.paging = create_models_pagination_example()
    return instance


def create_models_member_request_group_response_v1_example() -> ModelsMemberRequestGroupResponseV1:
    instance = ModelsMemberRequestGroupResponseV1()
    instance.group_id = randomize()
    instance.user_id = randomize("uid")
    return instance


def create_models_member_request_response_v1_example() -> ModelsMemberRequestResponseV1:
    instance = ModelsMemberRequestResponseV1()
    instance.group_id = randomize()
    instance.request_type = randomize()
    instance.user_id = randomize("uid")
    return instance


def create_models_pagination_example() -> ModelsPagination:
    instance = ModelsPagination()
    instance.first = randomize()
    instance.last = randomize()
    instance.next_ = randomize()
    instance.previous = randomize()
    return instance


def create_models_public_create_new_group_request_v1_example() -> ModelsPublicCreateNewGroupRequestV1:
    instance = ModelsPublicCreateNewGroupRequestV1()
    instance.configuration_code = randomize()
    instance.custom_attributes = {randomize(): randomize()}
    instance.group_description = randomize()
    instance.group_icon = randomize()
    instance.group_max_member = randomize("int", min_val=1, max_val=1000)
    instance.group_name = randomize()
    instance.group_region = randomize()
    instance.group_rules = create_models_group_rule_example()
    instance.group_type = randomize()
    return instance


def create_models_remove_role_from_member_request_v1_example() -> ModelsRemoveRoleFromMemberRequestV1:
    instance = ModelsRemoveRoleFromMemberRequestV1()
    instance.user_id = randomize("uid")
    return instance


def create_models_role_permission_example() -> ModelsRolePermission:
    instance = ModelsRolePermission()
    instance.action = randomize("int", min_val=1, max_val=1000)
    instance.resource_name = randomize()
    return instance


def create_models_rule_example() -> ModelsRule:
    instance = ModelsRule()
    instance.allowed_action = randomize()
    instance.rule_detail = [create_models_rule_information_example()]
    return instance


def create_models_rule_information_example() -> ModelsRuleInformation:
    instance = ModelsRuleInformation()
    instance.rule_attribute = randomize()
    instance.rule_criteria = randomize()
    instance.rule_value = randomize("int", min_val=1, max_val=1000)
    return instance


def create_models_update_group_configuration_global_rules_request_v1_example() -> ModelsUpdateGroupConfigurationGlobalRulesRequestV1:
    instance = ModelsUpdateGroupConfigurationGlobalRulesRequestV1()
    instance.rule_detail = [create_models_rule_information_example()]
    return instance


def create_models_update_group_configuration_request_v1_example() -> ModelsUpdateGroupConfigurationRequestV1:
    instance = ModelsUpdateGroupConfigurationRequestV1()
    instance.description = randomize()
    instance.group_max_member = randomize("int", min_val=1, max_val=1000)
    instance.name = randomize()
    return instance


def create_models_update_group_configuration_response_v1_example() -> ModelsUpdateGroupConfigurationResponseV1:
    instance = ModelsUpdateGroupConfigurationResponseV1()
    instance.configuration_code = randomize()
    instance.description = randomize()
    instance.global_rules = [create_models_rule_example()]
    instance.group_admin_role_id = randomize()
    instance.group_max_member = randomize("int", min_val=1, max_val=1000)
    instance.group_member_role_id = randomize()
    instance.name = randomize()
    return instance


def create_models_update_group_custom_attributes_request_v1_example() -> ModelsUpdateGroupCustomAttributesRequestV1:
    instance = ModelsUpdateGroupCustomAttributesRequestV1()
    instance.custom_attributes = {randomize(): randomize()}
    return instance


def create_models_update_group_custom_rule_request_v1_example() -> ModelsUpdateGroupCustomRuleRequestV1:
    instance = ModelsUpdateGroupCustomRuleRequestV1()
    instance.group_custom_rule = {randomize(): randomize()}
    return instance


def create_models_update_group_predefined_rule_request_v1_example() -> ModelsUpdateGroupPredefinedRuleRequestV1:
    instance = ModelsUpdateGroupPredefinedRuleRequestV1()
    instance.rule_detail = [create_models_rule_information_example()]
    return instance


def create_models_update_group_request_v1_example() -> ModelsUpdateGroupRequestV1:
    instance = ModelsUpdateGroupRequestV1()
    instance.custom_attributes = (
        create_models_update_group_request_v1_custom_attributes_example()
    )
    instance.group_description = randomize()
    instance.group_icon = randomize()
    instance.group_name = randomize()
    instance.group_region = randomize()
    instance.group_type = randomize()
    return instance


def create_models_update_group_request_v1_custom_attributes_example() -> ModelsUpdateGroupRequestV1CustomAttributes:
    instance = ModelsUpdateGroupRequestV1CustomAttributes()
    return instance


def create_models_update_member_role_permissions_request_v1_example() -> ModelsUpdateMemberRolePermissionsRequestV1:
    instance = ModelsUpdateMemberRolePermissionsRequestV1()
    instance.member_role_permissions = [create_models_role_permission_example()]
    return instance


def create_models_update_member_role_request_v1_example() -> ModelsUpdateMemberRoleRequestV1:
    instance = ModelsUpdateMemberRoleRequestV1()
    instance.member_role_name = randomize()
    return instance


def create_models_update_member_role_response_v1_example() -> ModelsUpdateMemberRoleResponseV1:
    instance = ModelsUpdateMemberRoleResponseV1()
    instance.member_role_id = randomize()
    instance.member_role_name = randomize()
    instance.member_role_permissions = [create_models_role_permission_example()]
    return instance


def create_models_user_invitation_response_v1_example() -> ModelsUserInvitationResponseV1:
    instance = ModelsUserInvitationResponseV1()
    instance.group_id = randomize()
    instance.user_id = randomize("uid")
    return instance


def create_response_error_response_example() -> ResponseErrorResponse:
    instance = ResponseErrorResponse()
    instance.error_code = randomize("int", min_val=1, max_val=1000)
    instance.error_message = randomize()
    return instance



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: accelbyte_cloud_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

# AccelByte Cloud Iam Service (5.22.0)

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from .....core import Operation
from .....core import HeaderStr
from .....core import HttpResponse
from .....core import StrEnum

from ...models import OauthmodelErrorResponse
from ...models import OauthmodelTokenWithDeviceCookieResponseV3


class GrantTypeEnum(StrEnum):
    AUTHORIZATION_CODE = "authorization_code"
    CLIENT_CREDENTIALS = "client_credentials"
    PASSWORD = "password"
    REFRESH_TOKEN = "refresh_token"


class TokenGrantV3(Operation):
    """OAuth2 access token generation endpoint (TokenGrantV3)

    This endpoint supports grant type:



      1. Grant Type == `authorization_code`:

        It generates the user token by given the authorization
    code which generated in "/iam/v3/authenticate" API response. It should also pass
    in the redirect_uri, which should be the same as generating the
    authorization code request.



      2. Grant Type == `password`:

        The grant type to use for authenticating a user, whether it's by email / username and password combination
    or through platform.



      3. Grant Type == `refresh_token`:

        Used to get a new access token for a valid refresh token.



      4. Grant Type == `client_credentials`:

        It generates a token by checking the client credentials provided through Authorization header.





    ## Access Token Content




    Following is the access tokenâs content:






      *


    namespace. It is the namespace the token was generated from.





      *


    display_name. The display name of the sub. It is empty if the token is generated from the client credential





      *


    roles. The subâs roles. It is empty if the token is generated from the client credential





      *


    namespace_roles. The subâs roles scoped to namespace. Improvement from roles, which make the role scoped to specific namespace instead of global to publisher namespace





      *


    permissions. The sub or audâ permissions





      *


    bans. The subâs list of bans. It is used by the IAM client for validating the token.





      *


    jflgs. It stands for Justice Flags. It is a special flag used for storing additional status information regarding the sub. It is implemented as a bit mask. Following explains what each bit represents:




        * 1: Email Address Verified



        * 2: Phone Number Verified



        * 4: Anonymous



        * 8: Suspicious Login






      *


    aud. The aud is the targeted resource server.





      *


    iat. The time the token issues at. It is in Epoch time format





      *


    exp. The time the token expires. It is in Epoch time format





      *


    client_id. The UserID. The sub is omitted if the token is generated from client credential





      *


    scope. The scope of the access request, expressed as a list of space-delimited, case-sensitive strings








    ## Bans




    The JWT contains user's active bans with its expiry date. List of ban types can be obtained from /bans.




    ## Device Cookie Validation




    For grant type "password" only




    Device Cookie is used to protect the user account from brute force login attack, [more detail from OWASP.




    This endpoint will read device cookie from request header Auth-Trust-Id. If device cookie not found, it will generate a new one and set it into response body auth_trust_id when successfully login.




    ## Track Login History




    This endpoint will track login history to detect suspicious login activity, please provide Device-Id (alphanumeric) in request header parameter otherwise it will set to "unknown".




    Align with General Data Protection Regulation in Europe, user login history will be kept within 28 days by default"




    ## 2FA remember device




    To remember device for 2FA, should provide cookie: device_token or header: Device-Token




    action code: 10703

    Properties:
        url: /iam/v3/oauth/token

        method: POST

        tags: ["OAuth2.0"]

        consumes: ["application/x-www-form-urlencoded"]

        produces: ["application/json"]

        securities: [BASIC_AUTH]

        auth_trust_id: (Auth-Trust-Id) OPTIONAL Union[str, HeaderStr] in header

        device_id: (device_id) OPTIONAL Union[str, HeaderStr] in header

        client_id: (client_id) OPTIONAL str in form_data

        code: (code) OPTIONAL str in form_data

        code_verifier: (code_verifier) OPTIONAL str in form_data

        extend_exp: (extend_exp) OPTIONAL bool in form_data

        password: (password) OPTIONAL str in form_data

        redirect_uri: (redirect_uri) OPTIONAL str in form_data

        refresh_token: (refresh_token) OPTIONAL str in form_data

        username: (username) OPTIONAL str in form_data

        grant_type: (grant_type) REQUIRED Union[str, GrantTypeEnum] in form_data

    Responses:
        200: OK - OauthmodelTokenWithDeviceCookieResponseV3 (Token returned)

        400: Bad Request - OauthmodelErrorResponse (InvalidRequest)

        401: Unauthorized - OauthmodelErrorResponse (Client authentication failed)

        403: Forbidden - OauthmodelErrorResponse (Unauthorized access)

        429: Too Many Requests - OauthmodelErrorResponse (Too many failed auth attempt)
    """

    # region fields

    _url: str = "/iam/v3/oauth/token"
    _method: str = "POST"
    _consumes: List[str] = ["application/x-www-form-urlencoded"]
    _produces: List[str] = ["application/json"]
    _securities: List[List[str]] = [["BASIC_AUTH"]]
    _location_query: str = None

    auth_trust_id: Union[str, HeaderStr]  # OPTIONAL in [header]
    device_id: Union[str, HeaderStr]  # OPTIONAL in [header]
    client_id: str  # OPTIONAL in [form_data]
    code: str  # OPTIONAL in [form_data]
    code_verifier: str  # OPTIONAL in [form_data]
    extend_exp: bool  # OPTIONAL in [form_data]
    password: str  # OPTIONAL in [form_data]
    redirect_uri: str  # OPTIONAL in [form_data]
    refresh_token: str  # OPTIONAL in [form_data]
    username: str  # OPTIONAL in [form_data]
    grant_type: Union[str, GrantTypeEnum]  # REQUIRED in [form_data]

    # endregion fields

    # region properties

    @property
    def url(self) -> str:
        return self._url

    @property
    def method(self) -> str:
        return self._method

    @property
    def consumes(self) -> List[str]:
        return self._consumes

    @property
    def produces(self) -> List[str]:
        return self._produces

    @property
    def securities(self) -> List[List[str]]:
        return self._securities

    @property
    def location_query(self) -> str:
        return self._location_query

    # endregion properties

    # region get methods

    # endregion get methods

    # region get_x_params methods

    def get_all_params(self) -> dict:
        return {
            "header": self.get_header_params(),
            "form_data": self.get_form_data_params(),
        }

    def get_header_params(self) -> dict:
        result = {}
        if hasattr(self, "auth_trust_id"):
            result["Auth-Trust-Id"] = self.auth_trust_id
        if hasattr(self, "device_id"):
            result["device_id"] = self.device_id
        return result

    def get_form_data_params(self) -> dict:
        result = {}
        if hasattr(self, "client_id"):
            result["client_id"] = self.client_id
        if hasattr(self, "code"):
            result["code"] = self.code
        if hasattr(self, "code_verifier"):
            result["code_verifier"] = self.code_verifier
        if hasattr(self, "extend_exp"):
            result["extend_exp"] = self.extend_exp
        if hasattr(self, "password"):
            result["password"] = self.password
        if hasattr(self, "redirect_uri"):
            result["redirect_uri"] = self.redirect_uri
        if hasattr(self, "refresh_token"):
            result["refresh_token"] = self.refresh_token
        if hasattr(self, "username"):
            result["username"] = self.username
        if hasattr(self, "grant_type"):
            result["grant_type"] = self.grant_type
        return result

    # endregion get_x_params methods

    # region is/has methods

    # endregion is/has methods

    # region with_x methods

    def with_auth_trust_id(self, value: Union[str, HeaderStr]) -> TokenGrantV3:
        self.auth_trust_id = value
        return self

    def with_device_id(self, value: Union[str, HeaderStr]) -> TokenGrantV3:
        self.device_id = value
        return self

    def with_client_id(self, value: str) -> TokenGrantV3:
        self.client_id = value
        return self

    def with_code(self, value: str) -> TokenGrantV3:
        self.code = value
        return self

    def with_code_verifier(self, value: str) -> TokenGrantV3:
        self.code_verifier = value
        return self

    def with_extend_exp(self, value: bool) -> TokenGrantV3:
        self.extend_exp = value
        return self

    def with_password(self, value: str) -> TokenGrantV3:
        self.password = value
        return self

    def with_redirect_uri(self, value: str) -> TokenGrantV3:
        self.redirect_uri = value
        return self

    def with_refresh_token(self, value: str) -> TokenGrantV3:
        self.refresh_token = value
        return self

    def with_username(self, value: str) -> TokenGrantV3:
        self.username = value
        return self

    def with_grant_type(self, value: Union[str, GrantTypeEnum]) -> TokenGrantV3:
        self.grant_type = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "auth_trust_id") and self.auth_trust_id:
            result["Auth-Trust-Id"] = str(self.auth_trust_id)
        elif include_empty:
            result["Auth-Trust-Id"] = ""
        if hasattr(self, "device_id") and self.device_id:
            result["device_id"] = str(self.device_id)
        elif include_empty:
            result["device_id"] = ""
        if hasattr(self, "client_id") and self.client_id:
            result["client_id"] = str(self.client_id)
        elif include_empty:
            result["client_id"] = ""
        if hasattr(self, "code") and self.code:
            result["code"] = str(self.code)
        elif include_empty:
            result["code"] = ""
        if hasattr(self, "code_verifier") and self.code_verifier:
            result["code_verifier"] = str(self.code_verifier)
        elif include_empty:
            result["code_verifier"] = ""
        if hasattr(self, "extend_exp") and self.extend_exp:
            result["extend_exp"] = bool(self.extend_exp)
        elif include_empty:
            result["extend_exp"] = False
        if hasattr(self, "password") and self.password:
            result["password"] = str(self.password)
        elif include_empty:
            result["password"] = ""
        if hasattr(self, "redirect_uri") and self.redirect_uri:
            result["redirect_uri"] = str(self.redirect_uri)
        elif include_empty:
            result["redirect_uri"] = ""
        if hasattr(self, "refresh_token") and self.refresh_token:
            result["refresh_token"] = str(self.refresh_token)
        elif include_empty:
            result["refresh_token"] = ""
        if hasattr(self, "username") and self.username:
            result["username"] = str(self.username)
        elif include_empty:
            result["username"] = ""
        if hasattr(self, "grant_type") and self.grant_type:
            result["grant_type"] = str(self.grant_type)
        elif include_empty:
            result["grant_type"] = Union[str, GrantTypeEnum]()
        return result

    # endregion to methods

    # region response methods

    # noinspection PyMethodMayBeStatic
    def parse_response(
        self, code: int, content_type: str, content: Any
    ) -> Tuple[
        Union[None, OauthmodelTokenWithDeviceCookieResponseV3],
        Union[None, HttpResponse, OauthmodelErrorResponse],
    ]:
        """Parse the given response.

        200: OK - OauthmodelTokenWithDeviceCookieResponseV3 (Token returned)

        400: Bad Request - OauthmodelErrorResponse (InvalidRequest)

        401: Unauthorized - OauthmodelErrorResponse (Client authentication failed)

        403: Forbidden - OauthmodelErrorResponse (Unauthorized access)

        429: Too Many Requests - OauthmodelErrorResponse (Too many failed auth attempt)

        ---: HttpResponse (Undocumented Response)

        ---: HttpResponse (Unexpected Content-Type Error)

        ---: HttpResponse (Unhandled Error)
        """
        pre_processed_response, error = self.pre_process_response(
            code=code, content_type=content_type, content=content
        )
        if error is not None:
            return None, None if error.is_no_content() else error
        code, content_type, content = pre_processed_response

        if code == 200:
            return (
                OauthmodelTokenWithDeviceCookieResponseV3.create_from_dict(content),
                None,
            )
        if code == 400:
            return None, OauthmodelErrorResponse.create_from_dict(content)
        if code == 401:
            return None, OauthmodelErrorResponse.create_from_dict(content)
        if code == 403:
            return None, OauthmodelErrorResponse.create_from_dict(content)
        if code == 429:
            return None, OauthmodelErrorResponse.create_from_dict(content)

        return self.handle_undocumented_response(
            code=code, content_type=content_type, content=content
        )

    # endregion response methods

    # region static methods

    @classmethod
    def create(
        cls,
        grant_type: Union[str, GrantTypeEnum],
        auth_trust_id: Optional[Union[str, HeaderStr]] = None,
        device_id: Optional[Union[str, HeaderStr]] = None,
        client_id: Optional[str] = None,
        code: Optional[str] = None,
        code_verifier: Optional[str] = None,
        extend_exp: Optional[bool] = None,
        password: Optional[str] = None,
        redirect_uri: Optional[str] = None,
        refresh_token: Optional[str] = None,
        username: Optional[str] = None,
    ) -> TokenGrantV3:
        instance = cls()
        instance.grant_type = grant_type
        if auth_trust_id is not None:
            instance.auth_trust_id = auth_trust_id
        if device_id is not None:
            instance.device_id = device_id
        if client_id is not None:
            instance.client_id = client_id
        if code is not None:
            instance.code = code
        if code_verifier is not None:
            instance.code_verifier = code_verifier
        if extend_exp is not None:
            instance.extend_exp = extend_exp
        if password is not None:
            instance.password = password
        if redirect_uri is not None:
            instance.redirect_uri = redirect_uri
        if refresh_token is not None:
            instance.refresh_token = refresh_token
        if username is not None:
            instance.username = username
        return instance

    @classmethod
    def create_from_dict(cls, dict_: dict, include_empty: bool = False) -> TokenGrantV3:
        instance = cls()
        if "Auth-Trust-Id" in dict_ and dict_["Auth-Trust-Id"] is not None:
            instance.auth_trust_id = str(dict_["Auth-Trust-Id"])
        elif include_empty:
            instance.auth_trust_id = ""
        if "device_id" in dict_ and dict_["device_id"] is not None:
            instance.device_id = str(dict_["device_id"])
        elif include_empty:
            instance.device_id = ""
        if "client_id" in dict_ and dict_["client_id"] is not None:
            instance.client_id = str(dict_["client_id"])
        elif include_empty:
            instance.client_id = ""
        if "code" in dict_ and dict_["code"] is not None:
            instance.code = str(dict_["code"])
        elif include_empty:
            instance.code = ""
        if "code_verifier" in dict_ and dict_["code_verifier"] is not None:
            instance.code_verifier = str(dict_["code_verifier"])
        elif include_empty:
            instance.code_verifier = ""
        if "extend_exp" in dict_ and dict_["extend_exp"] is not None:
            instance.extend_exp = bool(dict_["extend_exp"])
        elif include_empty:
            instance.extend_exp = False
        if "password" in dict_ and dict_["password"] is not None:
            instance.password = str(dict_["password"])
        elif include_empty:
            instance.password = ""
        if "redirect_uri" in dict_ and dict_["redirect_uri"] is not None:
            instance.redirect_uri = str(dict_["redirect_uri"])
        elif include_empty:
            instance.redirect_uri = ""
        if "refresh_token" in dict_ and dict_["refresh_token"] is not None:
            instance.refresh_token = str(dict_["refresh_token"])
        elif include_empty:
            instance.refresh_token = ""
        if "username" in dict_ and dict_["username"] is not None:
            instance.username = str(dict_["username"])
        elif include_empty:
            instance.username = ""
        if "grant_type" in dict_ and dict_["grant_type"] is not None:
            instance.grant_type = str(dict_["grant_type"])
        elif include_empty:
            instance.grant_type = Union[str, GrantTypeEnum]()
        return instance

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "Auth-Trust-Id": "auth_trust_id",
            "device_id": "device_id",
            "client_id": "client_id",
            "code": "code",
            "code_verifier": "code_verifier",
            "extend_exp": "extend_exp",
            "password": "password",
            "redirect_uri": "redirect_uri",
            "refresh_token": "refresh_token",
            "username": "username",
            "grant_type": "grant_type",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "Auth-Trust-Id": False,
            "device_id": False,
            "client_id": False,
            "code": False,
            "code_verifier": False,
            "extend_exp": False,
            "password": False,
            "redirect_uri": False,
            "refresh_token": False,
            "username": False,
            "grant_type": True,
        }

    @staticmethod
    def get_enum_map() -> Dict[str, List[Any]]:
        return {
            "grant_type": [
                "authorization_code",
                "client_credentials",
                "password",
                "refresh_token",
            ],  # in form_data
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: accelbyte_cloud_py_codegen

"""Auto-generated package that contains models used by the AccelByte Cloud Match Service V2."""

__version__ = "1.4.1"
__author__ = "AccelByte"
__email__ = "dev@accelbyte.net"

# pylint: disable=line-too-long

from .create_rule_set import CreateRuleSet
from .delete_rule_set import DeleteRuleSet
from .rule_set_details import RuleSetDetails
from .rule_set_list import RuleSetList
from .update_rule_set import UpdateRuleSet



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: accelbyte_cloud_py_codegen

# AccelByte Cloud Matchmaking Service (2.18.0)

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model

from ..models.models_matching_ally import ModelsMatchingAlly


class ServiceGetSessionHistoryDetailedResponseItem(Model):
    """Service get session history detailed response item (service.GetSessionHistoryDetailedResponseItem)

    Properties:
        channel: (channel) REQUIRED str

        client_version: (client_version) REQUIRED str

        created_at: (created_at) REQUIRED str

        event_description: (event_description) REQUIRED str

        event_name: (event_name) REQUIRED str

        game_mode: (game_mode) REQUIRED str

        id_: (_id) REQUIRED str

        joinable: (joinable) REQUIRED bool

        match_id: (match_id) REQUIRED str

        matching_allies: (matching_allies) REQUIRED List[ModelsMatchingAlly]

        namespace: (namespace) REQUIRED str

        region: (region) REQUIRED str

        server_name: (server_name) REQUIRED str

        status: (status) REQUIRED str

        party_id: (party_id) OPTIONAL str
    """

    # region fields

    channel: str  # REQUIRED
    client_version: str  # REQUIRED
    created_at: str  # REQUIRED
    event_description: str  # REQUIRED
    event_name: str  # REQUIRED
    game_mode: str  # REQUIRED
    id_: str  # REQUIRED
    joinable: bool  # REQUIRED
    match_id: str  # REQUIRED
    matching_allies: List[ModelsMatchingAlly]  # REQUIRED
    namespace: str  # REQUIRED
    region: str  # REQUIRED
    server_name: str  # REQUIRED
    status: str  # REQUIRED
    party_id: str  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_channel(self, value: str) -> ServiceGetSessionHistoryDetailedResponseItem:
        self.channel = value
        return self

    def with_client_version(
        self, value: str
    ) -> ServiceGetSessionHistoryDetailedResponseItem:
        self.client_version = value
        return self

    def with_created_at(
        self, value: str
    ) -> ServiceGetSessionHistoryDetailedResponseItem:
        self.created_at = value
        return self

    def with_event_description(
        self, value: str
    ) -> ServiceGetSessionHistoryDetailedResponseItem:
        self.event_description = value
        return self

    def with_event_name(
        self, value: str
    ) -> ServiceGetSessionHistoryDetailedResponseItem:
        self.event_name = value
        return self

    def with_game_mode(
        self, value: str
    ) -> ServiceGetSessionHistoryDetailedResponseItem:
        self.game_mode = value
        return self

    def with_id(self, value: str) -> ServiceGetSessionHistoryDetailedResponseItem:
        self.id_ = value
        return self

    def with_joinable(
        self, value: bool
    ) -> ServiceGetSessionHistoryDetailedResponseItem:
        self.joinable = value
        return self

    def with_match_id(self, value: str) -> ServiceGetSessionHistoryDetailedResponseItem:
        self.match_id = value
        return self

    def with_matching_allies(
        self, value: List[ModelsMatchingAlly]
    ) -> ServiceGetSessionHistoryDetailedResponseItem:
        self.matching_allies = value
        return self

    def with_namespace(
        self, value: str
    ) -> ServiceGetSessionHistoryDetailedResponseItem:
        self.namespace = value
        return self

    def with_region(self, value: str) -> ServiceGetSessionHistoryDetailedResponseItem:
        self.region = value
        return self

    def with_server_name(
        self, value: str
    ) -> ServiceGetSessionHistoryDetailedResponseItem:
        self.server_name = value
        return self

    def with_status(self, value: str) -> ServiceGetSessionHistoryDetailedResponseItem:
        self.status = value
        return self

    def with_party_id(self, value: str) -> ServiceGetSessionHistoryDetailedResponseItem:
        self.party_id = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "channel"):
            result["channel"] = str(self.channel)
        elif include_empty:
            result["channel"] = ""
        if hasattr(self, "client_version"):
            result["client_version"] = str(self.client_version)
        elif include_empty:
            result["client_version"] = ""
        if hasattr(self, "created_at"):
            result["created_at"] = str(self.created_at)
        elif include_empty:
            result["created_at"] = ""
        if hasattr(self, "event_description"):
            result["event_description"] = str(self.event_description)
        elif include_empty:
            result["event_description"] = ""
        if hasattr(self, "event_name"):
            result["event_name"] = str(self.event_name)
        elif include_empty:
            result["event_name"] = ""
        if hasattr(self, "game_mode"):
            result["game_mode"] = str(self.game_mode)
        elif include_empty:
            result["game_mode"] = ""
        if hasattr(self, "id_"):
            result["_id"] = str(self.id_)
        elif include_empty:
            result["_id"] = ""
        if hasattr(self, "joinable"):
            result["joinable"] = bool(self.joinable)
        elif include_empty:
            result["joinable"] = False
        if hasattr(self, "match_id"):
            result["match_id"] = str(self.match_id)
        elif include_empty:
            result["match_id"] = ""
        if hasattr(self, "matching_allies"):
            result["matching_allies"] = [
                i0.to_dict(include_empty=include_empty) for i0 in self.matching_allies
            ]
        elif include_empty:
            result["matching_allies"] = []
        if hasattr(self, "namespace"):
            result["namespace"] = str(self.namespace)
        elif include_empty:
            result["namespace"] = ""
        if hasattr(self, "region"):
            result["region"] = str(self.region)
        elif include_empty:
            result["region"] = ""
        if hasattr(self, "server_name"):
            result["server_name"] = str(self.server_name)
        elif include_empty:
            result["server_name"] = ""
        if hasattr(self, "status"):
            result["status"] = str(self.status)
        elif include_empty:
            result["status"] = ""
        if hasattr(self, "party_id"):
            result["party_id"] = str(self.party_id)
        elif include_empty:
            result["party_id"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        channel: str,
        client_version: str,
        created_at: str,
        event_description: str,
        event_name: str,
        game_mode: str,
        id_: str,
        joinable: bool,
        match_id: str,
        matching_allies: List[ModelsMatchingAlly],
        namespace: str,
        region: str,
        server_name: str,
        status: str,
        party_id: Optional[str] = None,
    ) -> ServiceGetSessionHistoryDetailedResponseItem:
        instance = cls()
        instance.channel = channel
        instance.client_version = client_version
        instance.created_at = created_at
        instance.event_description = event_description
        instance.event_name = event_name
        instance.game_mode = game_mode
        instance.id_ = id_
        instance.joinable = joinable
        instance.match_id = match_id
        instance.matching_allies = matching_allies
        instance.namespace = namespace
        instance.region = region
        instance.server_name = server_name
        instance.status = status
        if party_id is not None:
            instance.party_id = party_id
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ServiceGetSessionHistoryDetailedResponseItem:
        instance = cls()
        if not dict_:
            return instance
        if "channel" in dict_ and dict_["channel"] is not None:
            instance.channel = str(dict_["channel"])
        elif include_empty:
            instance.channel = ""
        if "client_version" in dict_ and dict_["client_version"] is not None:
            instance.client_version = str(dict_["client_version"])
        elif include_empty:
            instance.client_version = ""
        if "created_at" in dict_ and dict_["created_at"] is not None:
            instance.created_at = str(dict_["created_at"])
        elif include_empty:
            instance.created_at = ""
        if "event_description" in dict_ and dict_["event_description"] is not None:
            instance.event_description = str(dict_["event_description"])
        elif include_empty:
            instance.event_description = ""
        if "event_name" in dict_ and dict_["event_name"] is not None:
            instance.event_name = str(dict_["event_name"])
        elif include_empty:
            instance.event_name = ""
        if "game_mode" in dict_ and dict_["game_mode"] is not None:
            instance.game_mode = str(dict_["game_mode"])
        elif include_empty:
            instance.game_mode = ""
        if "_id" in dict_ and dict_["_id"] is not None:
            instance.id_ = str(dict_["_id"])
        elif include_empty:
            instance.id_ = ""
        if "joinable" in dict_ and dict_["joinable"] is not None:
            instance.joinable = bool(dict_["joinable"])
        elif include_empty:
            instance.joinable = False
        if "match_id" in dict_ and dict_["match_id"] is not None:
            instance.match_id = str(dict_["match_id"])
        elif include_empty:
            instance.match_id = ""
        if "matching_allies" in dict_ and dict_["matching_allies"] is not None:
            instance.matching_allies = [
                ModelsMatchingAlly.create_from_dict(i0, include_empty=include_empty)
                for i0 in dict_["matching_allies"]
            ]
        elif include_empty:
            instance.matching_allies = []
        if "namespace" in dict_ and dict_["namespace"] is not None:
            instance.namespace = str(dict_["namespace"])
        elif include_empty:
            instance.namespace = ""
        if "region" in dict_ and dict_["region"] is not None:
            instance.region = str(dict_["region"])
        elif include_empty:
            instance.region = ""
        if "server_name" in dict_ and dict_["server_name"] is not None:
            instance.server_name = str(dict_["server_name"])
        elif include_empty:
            instance.server_name = ""
        if "status" in dict_ and dict_["status"] is not None:
            instance.status = str(dict_["status"])
        elif include_empty:
            instance.status = ""
        if "party_id" in dict_ and dict_["party_id"] is not None:
            instance.party_id = str(dict_["party_id"])
        elif include_empty:
            instance.party_id = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ServiceGetSessionHistoryDetailedResponseItem]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ServiceGetSessionHistoryDetailedResponseItem]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        ServiceGetSessionHistoryDetailedResponseItem,
        List[ServiceGetSessionHistoryDetailedResponseItem],
        Dict[Any, ServiceGetSessionHistoryDetailedResponseItem],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "channel": "channel",
            "client_version": "client_version",
            "created_at": "created_at",
            "event_description": "event_description",
            "event_name": "event_name",
            "game_mode": "game_mode",
            "_id": "id_",
            "joinable": "joinable",
            "match_id": "match_id",
            "matching_allies": "matching_allies",
            "namespace": "namespace",
            "region": "region",
            "server_name": "server_name",
            "status": "status",
            "party_id": "party_id",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "channel": True,
            "client_version": True,
            "created_at": True,
            "event_description": True,
            "event_name": True,
            "game_mode": True,
            "_id": True,
            "joinable": True,
            "match_id": True,
            "matching_allies": True,
            "namespace": True,
            "region": True,
            "server_name": True,
            "status": True,
            "party_id": False,
        }

    # endregion static methods
